@@ -1,6 +1,6 @@
 /*
-  Copyright(C) 2014-2018 Brazil
-  Copyright(C) 2018-2019 Kouhei Sutou <kou@clear-code.com>
+  Copyright(C) 2014-2018  Brazil
+  Copyright(C) 2018-2020  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -23,8 +23,10 @@
 
 #include "groonga/portability.h"
 #include "groonga/groonga.h"
+#include "groonga/posting.h"
 
 #include "groonga/accessor.h"
+#include "groonga/aggregator.h"
 #include "groonga/array.h"
 #include "groonga/arrow.h"
 #include "groonga/cache.h"
@@ -49,11 +51,15 @@
 #include "groonga/output.h"
 #include "groonga/output_columns.h"
 #include "groonga/pat.h"
+#include "groonga/proc.h"
 #include "groonga/raw_string.h"
 #include "groonga/request_canceler.h"
 #include "groonga/request_timer.h"
+#include "groonga/result_set.h"
+#include "groonga/selector.h"
 #include "groonga/string.h"
 #include "groonga/table.h"
+#include "groonga/table_module.h"
 #include "groonga/thread.h"
 #include "groonga/time.h"
 #include "groonga/token.h"
