@@ -1,6 +1,6 @@
 /*
-  Copyright(C) 2009-2017 Brazil
-  Copyright(C) 2019 Sutou Kouhei <kou@clear-code.com>
+  Copyright(C) 2009-2017  Brazil
+  Copyright(C) 2019-2020  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -23,12 +23,13 @@
 extern "C" {
 #endif
 
-typedef unsigned int grn_expr_flags;
+typedef uint32_t grn_expr_flags;
 
 #define GRN_EXPR_SYNTAX_QUERY          (0x00)
 #define GRN_EXPR_SYNTAX_SCRIPT         (0x01)
 #define GRN_EXPR_SYNTAX_OUTPUT_COLUMNS (0x20)
 #define GRN_EXPR_SYNTAX_ADJUSTER       (0x40)
+#define GRN_EXPR_SYNTAX_GROUP_KEYS     (0x100)
 #define GRN_EXPR_ALLOW_PRAGMA          (0x02)
 #define GRN_EXPR_ALLOW_COLUMN          (0x04)
 #define GRN_EXPR_ALLOW_UPDATE          (0x08)
@@ -139,6 +140,13 @@ grn_expr_set_parent(grn_ctx *ctx,
 GRN_API grn_obj *
 grn_expr_get_parent(grn_ctx *ctx, grn_obj *expr);
 
+GRN_API grn_rc
+grn_expr_set_condition(grn_ctx *ctx,
+                       grn_obj *expr,
+                       grn_obj *condition);
+GRN_API grn_obj *
+grn_expr_get_condition(grn_ctx *ctx, grn_obj *expr);
+
 #ifdef __cplusplus
 }
 #endif
