@@ -31,7 +31,7 @@ extern "C" {
   result = grn_obj_is_true(ctx, obj);           \
 } while (0)
 
-GRN_API grn_bool grn_obj_is_true(grn_ctx *ctx, grn_obj *obj);
+GRN_API bool grn_obj_is_true(grn_ctx *ctx, grn_obj *obj);
 
 GRN_API bool grn_obj_is_temporary(grn_ctx *ctx, grn_obj *obj);
 
@@ -44,9 +44,13 @@ GRN_API bool grn_obj_is_weight_vector(grn_ctx *ctx, grn_obj *obj);
 GRN_API bool grn_obj_is_uvector(grn_ctx *ctx, grn_obj *obj);
 GRN_API bool grn_obj_is_weight_uvector(grn_ctx *ctx, grn_obj *obj);
 GRN_API grn_bool grn_obj_is_table(grn_ctx *ctx, grn_obj *obj);
-GRN_API grn_bool grn_obj_is_lexicon(grn_ctx *ctx, grn_obj *obj);
+GRN_API bool grn_obj_is_table_with_key(grn_ctx *ctx, grn_obj *obj);
+GRN_API bool grn_obj_is_lexicon(grn_ctx *ctx, grn_obj *obj);
+GRN_API bool grn_obj_is_lexicon_without_data_column(grn_ctx *ctx, grn_obj *obj);
+GRN_API bool grn_obj_is_tiny_hash_table(grn_ctx *ctx, grn_obj *obj);
 GRN_API grn_bool grn_obj_is_column(grn_ctx *ctx, grn_obj *obj);
 GRN_API grn_bool grn_obj_is_scalar_column(grn_ctx *ctx, grn_obj *obj);
+GRN_API bool grn_obj_is_text_family_scalar_column(grn_ctx *ctx, grn_obj *obj);
 GRN_API bool grn_obj_is_number_family_scalar_column(grn_ctx *ctx, grn_obj *obj);
 GRN_API grn_bool grn_obj_is_vector_column(grn_ctx *ctx, grn_obj *obj);
 GRN_API grn_bool grn_obj_is_weight_vector_column(grn_ctx *ctx, grn_obj *obj);
@@ -120,6 +124,11 @@ grn_obj_get_option_values(grn_ctx *ctx,
                           grn_obj *values);
 GRN_API grn_rc grn_obj_clear_option_values(grn_ctx *ctx, grn_obj *obj);
 
+GRN_API grn_rc
+grn_obj_to_script_syntax(grn_ctx *ctx,
+                         grn_obj *obj,
+                         grn_obj *buffer);
+
 #ifdef __cplusplus
 }
 #endif
