@@ -1,5 +1,5 @@
 /*
-  Copyright(C) 2015-2016 Brazil
+  Copyright(C) 2015-2017 Brazil
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -181,3 +181,31 @@
 #else /* WIN32 */
 # define grn_close(fd) close((fd))
 #endif /* WIN32 */
+
+#ifdef WIN32
+# define grn_fileno(stream) _fileno((stream))
+#else /* WIN32 */
+# define grn_fileno(stream) fileno((stream))
+#endif /* WIN32 */
+
+#ifdef WIN32
+# define grn_isatty(stream) _isatty((stream))
+#else /* WIN32 */
+# define grn_isatty(stream) isatty((stream))
+#endif /* WIN32 */
+
+#ifdef WIN32
+# define grn_getpid() _getpid()
+#else /* WIN32 */
+# define grn_getpid() getpid()
+#endif /* WIN32 */
+
+#ifdef WIN32
+# define grn_time_t __time64_t
+# define grn_time(dest_time) _time64((dest_time))
+# define grn_mktime(tm) _mktime64((tm))
+#else /* WIN32 */
+# define grn_time_t time_t
+# define grn_time(dest_time) time((dest_time))
+# define grn_mktime(tm) mktime((tm))
+#endif /* WIN32 */
