@@ -82,9 +82,50 @@ GRN_API size_t
 grn_table_cursor_get_max_n_records(grn_ctx *ctx, grn_table_cursor *cursor);
 GRN_API grn_obj *grn_table_cursor_table(grn_ctx *ctx, grn_table_cursor *tc);
 
-GRN_API grn_obj *grn_index_cursor_open(grn_ctx *ctx, grn_table_cursor *tc, grn_obj *index,
-                                       grn_id rid_min, grn_id rid_max, int flags);
-GRN_API grn_posting *grn_index_cursor_next(grn_ctx *ctx, grn_obj *ic, grn_id *tid);
+GRN_API grn_obj *
+grn_index_cursor_open(grn_ctx *ctx,
+                      grn_table_cursor *tc,
+                      grn_obj *index,
+                      grn_id rid_min,
+                      grn_id rid_max,
+                      int flags);
+GRN_API grn_obj *
+grn_index_cursor_get_index_column(grn_ctx *ctx,
+                                  grn_obj *index_cursor);
+GRN_API grn_rc
+grn_index_cursor_set_term_id(grn_ctx *ctx,
+                             grn_obj *index_cursor,
+                             grn_id term_id);
+GRN_API grn_rc
+grn_index_cursor_set_section_id(grn_ctx *ctx,
+                                grn_obj *index_cursor,
+                                uint32_t section_id);
+GRN_API uint32_t
+grn_index_cursor_get_section_id(grn_ctx *ctx,
+                                grn_obj *index_cursor);
+GRN_API grn_rc
+grn_index_cursor_set_start_position(grn_ctx *ctx,
+                                    grn_obj *index_cursor,
+                                    uint32_t position);
+GRN_API grn_rc
+grn_index_cursor_reset_start_position(grn_ctx *ctx,
+                                      grn_obj *index_cursor);
+GRN_API uint32_t
+grn_index_cursor_get_start_position(grn_ctx *ctx,
+                                    grn_obj *index_cursor);
+GRN_API grn_posting *
+grn_index_cursor_next(grn_ctx *ctx,
+                      grn_obj *index_cursor,
+                      grn_id *term_id);
+
+typedef grn_rc (*grn_table_cursor_foreach_func)(grn_ctx *ctx,
+                                                grn_table_cursor *cursor,
+                                                grn_id id,
+                                                void *user_data);
+GRN_API grn_rc grn_table_cursor_foreach(grn_ctx *ctx,
+                                        grn_table_cursor *cursor,
+                                        grn_table_cursor_foreach_func func,
+                                        void *user_data);
 
 #define GRN_TABLE_EACH(ctx,table,head,tail,id,key,key_size,value,block) do {\
   (ctx)->errlvl = GRN_LOG_NOTICE;\
@@ -176,11 +217,60 @@ GRN_API int grn_table_sort(grn_ctx *ctx, grn_obj *table, int offset, int limit,
 typedef struct _grn_table_group_result grn_table_group_result;
 typedef uint32_t grn_table_group_flags;
 
-#define GRN_TABLE_GROUP_CALC_COUNT     (0x01<<3)
-#define GRN_TABLE_GROUP_CALC_MAX       (0x01<<4)
-#define GRN_TABLE_GROUP_CALC_MIN       (0x01<<5)
-#define GRN_TABLE_GROUP_CALC_SUM       (0x01<<6)
-#define GRN_TABLE_GROUP_CALC_AVG       (0x01<<7)
+#define GRN_TABLE_GROUP_CALC_COUNT      (0x01<<3)
+#define GRN_TABLE_GROUP_CALC_MAX        (0x01<<4)
+#define GRN_TABLE_GROUP_CALC_MIN        (0x01<<5)
+#define GRN_TABLE_GROUP_CALC_SUM        (0x01<<6)
+/* Deprecated since 10.0.4. Use GRN_TABLE_GROUP_CALC_MEAN instead. */
+#define GRN_TABLE_GROUP_CALC_AVG        GRN_TABLE_GROUP_CALC_MEAN
+#define GRN_TABLE_GROUP_CALC_MEAN       (0x01<<7)
+#define GRN_TABLE_GROUP_CALC_AGGREGATOR (0x01<<8)
+
+typedef struct _grn_table_group_aggregator grn_table_group_aggregator;
+
+GRN_API grn_table_group_aggregator *
+grn_table_group_aggregator_open(grn_ctx *ctx);
+GRN_API grn_rc
+grn_table_group_aggregator_close(grn_ctx *ctx,
+                                 grn_table_group_aggregator *aggregator);
+GRN_API grn_rc
+grn_table_group_aggregator_set_output_column_name(
+  grn_ctx *ctx,
+  grn_table_group_aggregator *aggregator,
+  const char *name,
+  int32_t name_len);
+GRN_API const char *
+grn_table_group_aggregator_get_output_column_name(
+  grn_ctx *ctx,
+  grn_table_group_aggregator *aggregator,
+  uint32_t *len);
+GRN_API grn_rc
+grn_table_group_aggregator_set_output_column_type(
+  grn_ctx *ctx,
+  grn_table_group_aggregator *aggregator,
+  grn_obj *type);
+GRN_API grn_obj *
+grn_table_group_aggregator_get_output_column_type(
+  grn_ctx *ctx,
+  grn_table_group_aggregator *aggregator);
+GRN_API grn_rc
+grn_table_group_aggregator_set_output_column_flags(
+  grn_ctx *ctx,
+  grn_table_group_aggregator *aggregator,
+  grn_column_flags flags);
+GRN_API grn_column_flags
+grn_table_group_aggregator_get_output_column_flags(
+  grn_ctx *ctx,
+  grn_table_group_aggregator *aggregator);
+GRN_API grn_rc
+grn_table_group_aggregator_set_expression(grn_ctx *ctx,
+                                          grn_table_group_aggregator *aggregator,
+                                          const char *expression,
+                                          int32_t expression_len);
+GRN_API const char *
+grn_table_group_aggregator_get_expression(grn_ctx *ctx,
+                                          grn_table_group_aggregator *aggregator,
+                                          uint32_t *expression_len);
 
 struct _grn_table_group_result {
   grn_obj *table;
@@ -191,11 +281,19 @@ struct _grn_table_group_result {
   grn_operator op;
   unsigned int max_n_subrecs;
   grn_obj *calc_target;
+  grn_table_group_aggregator **aggregators;
+  uint32_t n_aggregators;
 };
 
 GRN_API grn_rc grn_table_group(grn_ctx *ctx, grn_obj *table,
                                grn_table_sort_key *keys, int n_keys,
                                grn_table_group_result *results, int n_results);
+GRN_API grn_table_sort_key *
+grn_table_group_keys_parse(grn_ctx *ctx,
+                           grn_obj *table,
+                           const char *raw_sort_keys,
+                           int32_t raw_sort_keys_size,
+                           int *n_keys);
 GRN_API grn_rc grn_table_setoperation(grn_ctx *ctx, grn_obj *table1, grn_obj *table2,
                                       grn_obj *res, grn_operator op);
 GRN_API grn_rc grn_table_difference(grn_ctx *ctx, grn_obj *table1, grn_obj *table2,
@@ -245,92 +343,17 @@ GRN_API grn_rc grn_table_apply_expr(grn_ctx *ctx,
 
 GRN_API grn_id grn_table_find_reference_object(grn_ctx *ctx, grn_obj *table);
 
-typedef void *(*grn_table_module_open_options_func)(grn_ctx *ctx,
-                                                    grn_obj *proc,
-                                                    grn_obj *values,
-                                                    void *user_data);
-/* Deprecated since 8.0.9. Use grn_table_module_option_options_func instead. */
-typedef grn_table_module_open_options_func grn_tokenizer_open_options_func;
-
-GRN_API grn_rc
-grn_table_set_default_tokenizer_options(grn_ctx *ctx,
-                                        grn_obj *table,
-                                        grn_obj *options);
-
-GRN_API grn_rc
-grn_table_get_default_tokenizer_options(grn_ctx *ctx,
-                                        grn_obj *table,
-                                        grn_obj *options);
-
-GRN_API void *
-grn_table_cache_default_tokenizer_options(grn_ctx *ctx,
-                                          grn_obj *table,
-                                          grn_table_module_open_options_func open_options_func,
-                                          grn_close_func close_options_func,
-                                          void *user_data);
-
-GRN_API grn_rc
-grn_table_get_default_tokenizer_string(grn_ctx *ctx,
-                                       grn_obj *table,
-                                       grn_obj *output);
-
-
-/* Deprecated since 8.0.9. Use grn_table_module_open_options_func instead. */
-typedef grn_table_module_open_options_func grn_normalizer_open_options_func;
-
-GRN_API grn_rc
-grn_table_set_normalizer_options(grn_ctx *ctx,
-                                 grn_obj *table,
-                                 grn_obj *options);
-
-GRN_API grn_rc
-grn_table_get_normalizer_options(grn_ctx *ctx,
-                                 grn_obj *table,
-                                 grn_obj *options);
-
-/* TODO: Remove string argument. It's needless. */
-GRN_API void *
-grn_table_cache_normalizer_options(grn_ctx *ctx,
-                                   grn_obj *table,
-                                   grn_obj *string,
-                                   grn_table_module_open_options_func open_options_func,
-                                   grn_close_func close_options_func,
-                                   void *user_data);
-
-GRN_API grn_rc
-grn_table_get_normalizer_string(grn_ctx *ctx,
-                                grn_obj *table,
-                                grn_obj *output);
-
-
-GRN_API grn_rc
-grn_table_set_token_filter_options(grn_ctx *ctx,
-                                   grn_obj *table,
-                                   unsigned int i,
-                                   grn_obj *options);
-
-GRN_API grn_rc
-grn_table_get_token_filter_options(grn_ctx *ctx,
-                                   grn_obj *table,
-                                   unsigned int i,
-                                   grn_obj *options);
-
-GRN_API void *
-grn_table_cache_token_filter_options(grn_ctx *ctx,
-                                     grn_obj *table,
-                                     unsigned int i,
-                                     grn_table_module_open_options_func open_options_func,
-                                     grn_close_func close_options_func,
-                                     void *user_data);
-
-GRN_API grn_rc
-grn_table_get_token_filters_string(grn_ctx *ctx,
-                                   grn_obj *table,
-                                   grn_obj *output);
-
 GRN_API grn_rc
 grn_table_copy(grn_ctx *ctx, grn_obj *from, grn_obj *to);
 
+GRN_API grn_rc
+grn_table_get_duplicated_keys(grn_ctx *ctx,
+                              grn_obj *table,
+                              grn_obj **duplicated_keys);
+GRN_API bool
+grn_table_have_duplicated_keys(grn_ctx *ctx,
+                               grn_obj *table);
+
 #ifdef __cplusplus
 }
 #endif
