@@ -1,7 +1,7 @@
 /* -*- c-basic-offset: 2 -*- */
 /*
-  Copyright(C) 2009-2016 Brazil
-  Copyright(C) 2018 Kouhei Sutou <kou@clear-code.com>
+  Copyright(C) 2009-2016  Brazil
+  Copyright(C) 2018-2020  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -43,6 +43,19 @@ grn_token_cursor_open(grn_ctx *ctx,
                       grn_tokenize_mode mode,
                       uint32_t flags);
 
+GRN_API grn_rc
+grn_token_cursor_set_source_column(grn_ctx *ctx,
+                                   grn_token_cursor *token_cursor,
+                                   grn_obj *column);
+GRN_API grn_rc
+grn_token_cursor_set_source_id(grn_ctx *ctx,
+                               grn_token_cursor *token_cursor,
+                               grn_id id);
+GRN_API grn_rc
+grn_token_cursor_set_index_column(grn_ctx *ctx,
+                                  grn_token_cursor *token_cursor,
+                                  grn_obj *column);
+
 GRN_API grn_id
 grn_token_cursor_next(grn_ctx *ctx, grn_token_cursor *token_cursor);
 
