@@ -82,12 +82,18 @@ namespace grnarrow {
     }
   }
 
+  bool check(grn_ctx *ctx,
+             const arrow::Status &status,
+             const std::string &context) {
+    return check(ctx, status, context.c_str());
+  }
+
   bool check(grn_ctx *ctx,
              const arrow::Status &status,
              std::ostream &output) {
     return check(ctx,
                  status,
-                 static_cast<std::stringstream &>(output).str().c_str());
+                 static_cast<std::stringstream &>(output).str());
   }
 
   class ObjectCache {
@@ -195,7 +201,7 @@ namespace grnarrow {
         auto arrow_type = grn_type_id_to_arrow_type(ctx, range_id);
         if (!arrow_type) {
           auto range = object_cache[range_id];
-          if (grn_obj_is_lexicon(ctx, range)) {
+          if (grn_obj_is_table_with_key(ctx, range)) {
             // TODO: We can't return reference value as dictionary
             // because arrow::ipc::RecordBatchWriter doesn't support
             // dictionary delta for now.
@@ -1135,7 +1141,11 @@ namespace grnarrow {
 
       auto schema = std::make_shared<arrow::Schema>(fields);
 
+#if ARROW_VERSION_MAJOR >= 2
+      auto writer_result = arrow::ipc::MakeFileWriter(output, schema);
+#else
       auto writer_result = arrow::ipc::NewFileWriter(output, schema);
+#endif
       if (!check(ctx_,
                  writer_result,
                  "[arrow][dump] failed to create file format writer")) {
@@ -1770,7 +1780,11 @@ namespace grnarrow {
       schema_builder_.Reset();
       schema_ = *schema;
 
+#if ARROW_VERSION_MAJOR >= 2
+      auto writer = arrow::ipc::MakeStreamWriter(&output_, schema_);
+#else
       auto writer = arrow::ipc::NewStreamWriter(&output_, schema_);
+#endif
       if (!check(ctx_,
                  writer,
                  "[arrow][stream-writer][write-schema] "
@@ -1868,6 +1882,22 @@ namespace grnarrow {
             "failed to add a column value: <" << value << ">");
     }
 
+    void add_column_uint64(int64_t value) {
+      auto column_builder =
+        record_batch_builder_->GetFieldAs<arrow::UInt64Builder>(
+          current_column_index_++);
+      auto status = column_builder->Append(value);
+      if (!status.ok()) {
+        return;
+      }
+      std::stringstream context;
+      check(ctx_,
+            status,
+            context <<
+            "[arrow][stream-writer][add-column][uint64] " <<
+            "failed to add a column value: <" << value << ">");
+    }
+
     void add_column_timestamp(grn_timeval value) {
       auto column_builder =
         record_batch_builder_->GetFieldAs<arrow::TimestampBuilder>(
@@ -1954,7 +1984,7 @@ namespace grnarrow {
         auto element_size = grn_uvector_element_size(ctx_, uvector);
         auto n = GRN_BULK_VSIZE(uvector) / element_size;
         auto raw_value_builder = column_builder->value_builder();
-        if (grn_obj_is_lexicon(ctx_, domain)) {
+        if (grn_obj_is_table_with_key(ctx_, domain)) {
           auto value_builder =
             static_cast<arrow::StringBuilder *>(raw_value_builder);
           for (long i = 0; i < n; ++i) {
@@ -2035,6 +2065,218 @@ namespace grnarrow {
     ObjectCache object_cache_;
   };
 }
+
+namespace grn {
+  namespace arrow {
+    struct ArrayBuilder::Impl {
+      struct ColumnAppender : ::arrow::TypeVisitor {
+        grn_ctx *ctx_;
+        grn_obj *column_;
+        grn_table_cursor *cursor_;
+        ::arrow::ArrayBuilder *builder_;
+
+        ColumnAppender(grn_ctx *ctx,
+                       grn_obj *column,
+                       grn_table_cursor *cursor,
+                       ::arrow::ArrayBuilder *builder) :
+          ctx_(ctx),
+          column_(column),
+          cursor_(cursor),
+          builder_(builder) {
+        }
+
+#define VISIT(TYPE)                                             \
+        ::arrow::Status                                         \
+        Visit(const TYPE &type) override {                      \
+          return Append(type);                                  \
+        }
+
+        VISIT(::arrow::BooleanType)
+        VISIT(::arrow::Int8Type)
+        VISIT(::arrow::UInt8Type)
+        VISIT(::arrow::Int16Type)
+        VISIT(::arrow::UInt16Type)
+        VISIT(::arrow::Int32Type)
+        VISIT(::arrow::UInt32Type)
+        VISIT(::arrow::Int64Type)
+        VISIT(::arrow::UInt64Type)
+        VISIT(::arrow::FloatType)
+        VISIT(::arrow::DoubleType)
+        VISIT(::arrow::StringType)
+        VISIT(::arrow::TimestampType)
+
+#undef VISIT
+
+        template <typename Type>
+        ::arrow::Status Append(const Type& type) {
+          using Builder = typename ::arrow::TypeTraits<Type>::BuilderType;
+          using CType = typename ::arrow::TypeTraits<Type>::CType;
+          auto builder = static_cast<Builder *>(builder_);
+          grn_id id;
+          while ((id = grn_table_cursor_next(ctx_, cursor_)) != GRN_ID_NIL) {
+            uint32_t size;
+            auto raw_data = grn_obj_get_value_(ctx_, column_, id, &size);
+            const auto data = *reinterpret_cast<const CType *>(raw_data);
+            ARROW_RETURN_NOT_OK(builder->Append(data));
+          }
+          return ::arrow::Status::OK();
+        }
+
+        ::arrow::Status Append(const ::arrow::StringType& type) {
+          using Builder = ::arrow::StringBuilder;
+          auto builder = static_cast<Builder *>(builder_);
+          grn_id id;
+          while ((id = grn_table_cursor_next(ctx_, cursor_)) != GRN_ID_NIL) {
+            uint32_t size;
+            auto raw_data = grn_obj_get_value_(ctx_, column_, id, &size);
+            ARROW_RETURN_NOT_OK(builder->Append(raw_data, size));
+          }
+          return ::arrow::Status::OK();
+        }
+      };
+
+      grn_ctx *ctx_;
+      std::unique_ptr<::arrow::ArrayBuilder> builder_;
+      grnarrow::ObjectCache object_cache_;
+
+      Impl(grn_ctx *ctx) : ctx_(ctx),
+                           builder_(nullptr),
+                           object_cache_(ctx_) {
+      }
+
+      ::arrow::Status add_column(grn_obj *column,
+                                 grn_table_cursor *cursor) {
+        const auto arrow_type =
+          grn_column_to_arrow_type(ctx_, column, object_cache_);
+        if (!arrow_type) {
+          grn::TextBulk inspected(ctx_);
+          grn_inspect(ctx_, *inspected, column);
+          return ::arrow::Status::NotImplemented(
+            "[arrow][array-builder][add-column] "
+            "unsupported column: ", inspected.value());
+        }
+        if (!builder_) {
+          ARROW_RETURN_NOT_OK(
+            ::arrow::MakeBuilder(::arrow::default_memory_pool(),
+                                 arrow_type,
+                                 &builder_));
+        }
+        ColumnAppender appender(ctx_, column, cursor, builder_.get());
+        return arrow_type->Accept(&appender);
+      }
+
+      ::arrow::Result<std::shared_ptr<::arrow::Array>>
+      finish() {
+        return builder_->Finish();
+      }
+    };
+
+    ArrayBuilder::ArrayBuilder(grn_ctx *ctx) : impl_(new Impl(ctx)) {
+    }
+
+    ArrayBuilder::~ArrayBuilder() = default;
+
+    ::arrow::Status ArrayBuilder::add_column(grn_obj *column,
+                                             grn_table_cursor *cursor) {
+      return impl_->add_column(column, cursor);
+    }
+
+    ::arrow::Result<std::shared_ptr<::arrow::Array>>
+    ArrayBuilder::finish() {
+      return impl_->finish();
+    }
+
+    namespace {
+      class ArrayValueGetter : public ::arrow::ArrayVisitor {
+      public:
+        ArrayValueGetter(grn_ctx *ctx,
+                         int64_t index,
+                         grn_obj *value) :
+          ctx_(ctx),
+          index_(index),
+          value_(value) {
+        }
+
+        ::arrow::Status Visit(const ::arrow::BooleanArray& array) {
+          GRN_BOOL_PUT(ctx_, value_, array.Value(index_));
+          return ::arrow::Status::OK();
+        }
+
+        ::arrow::Status Visit(const ::arrow::Int8Array& array) {
+          GRN_INT8_PUT(ctx_, value_, array.Value(index_));
+          return ::arrow::Status::OK();
+        }
+
+        ::arrow::Status Visit(const ::arrow::UInt8Array& array) {
+          GRN_UINT8_PUT(ctx_, value_, array.Value(index_));
+          return ::arrow::Status::OK();
+        }
+
+        ::arrow::Status Visit(const ::arrow::Int16Array& array) {
+          GRN_INT16_PUT(ctx_, value_, array.Value(index_));
+          return ::arrow::Status::OK();
+        }
+
+        ::arrow::Status Visit(const ::arrow::UInt16Array& array) {
+          GRN_UINT16_PUT(ctx_, value_, array.Value(index_));
+          return ::arrow::Status::OK();
+        }
+
+        ::arrow::Status Visit(const ::arrow::Int32Array& array) {
+          GRN_INT32_PUT(ctx_, value_, array.Value(index_));
+          return ::arrow::Status::OK();
+        }
+
+        ::arrow::Status Visit(const ::arrow::UInt32Array& array) {
+          GRN_UINT32_PUT(ctx_, value_, array.Value(index_));
+          return ::arrow::Status::OK();
+        }
+
+        ::arrow::Status Visit(const ::arrow::Int64Array& array) {
+          GRN_INT64_PUT(ctx_, value_, array.Value(index_));
+          return ::arrow::Status::OK();
+        }
+
+        ::arrow::Status Visit(const ::arrow::UInt64Array& array) {
+          GRN_UINT64_PUT(ctx_, value_, array.Value(index_));
+          return ::arrow::Status::OK();
+        }
+
+        ::arrow::Status Visit(const ::arrow::TimestampArray& array) {
+          const auto &arrow_timestamp_type =
+            std::static_pointer_cast<::arrow::TimestampType>(array.type());
+          const auto time_unit = arrow_timestamp_type->unit();
+          grnarrow::put_time_value(ctx_,
+                                   value_,
+                                   array.Value(index_),
+                                   time_unit);
+          return ::arrow::Status::OK();
+        }
+
+        ::arrow::Status Visit(const ::arrow::StringArray& array) {
+          auto raw_value = array.GetView(index_);
+          grn_bulk_write(ctx_, value_, raw_value.data(), raw_value.length());
+          return ::arrow::Status::OK();
+        }
+
+      private:
+        grn_ctx *ctx_;
+        int64_t index_;
+        grn_obj *value_;
+      };
+    }
+
+    grn_rc get_value(grn_ctx *ctx,
+                     const ::arrow::Array *array,
+                     int64_t index,
+                     grn_obj *value) {
+      ArrayValueGetter getter(ctx, index, value);
+      auto status = array->Accept(&getter);
+      grnarrow::check(ctx, status, "[arrow][value][get] failed");
+      return ctx->rc;
+    }
+  }
+}
 #endif /* GRN_WITH_APACHE_ARROW */
 
 extern "C" {
@@ -2396,6 +2638,22 @@ grn_arrow_stream_writer_add_column_int64(grn_ctx *ctx,
   GRN_API_RETURN(ctx->rc);
 }
 
+grn_rc
+grn_arrow_stream_writer_add_column_uint64(grn_ctx *ctx,
+                                          grn_arrow_stream_writer *writer,
+                                          uint64_t value)
+{
+  GRN_API_ENTER;
+#ifdef GRN_WITH_APACHE_ARROW
+  writer->writer->add_column_uint64(value);
+#else
+  ERR(GRN_FUNCTION_NOT_IMPLEMENTED,
+      "[arrow][stream-writer][add-column][uint64] "
+      "Apache Arrow support isn't enabled");
+#endif
+  GRN_API_RETURN(ctx->rc);
+}
+
 grn_rc
 grn_arrow_stream_writer_add_column_timestamp(grn_ctx *ctx,
                                              grn_arrow_stream_writer *writer,
