@@ -1,5 +1,6 @@
 /* -*- c-basic-offset: 2 -*- */
-/* Copyright(C) 2011 Brazil
+/*
+  Copyright(C) 2011-2016 Brazil
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -12,18 +13,17 @@
 
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
-#ifndef GRN_DAT_HPP_HEADER_HPP_
-#define GRN_DAT_HPP_HEADER_HPP_
+#pragma once
 
 #include "dat.hpp"
 
 namespace grn {
 namespace dat {
 
-class Header {
+class GRN_DAT_API Header {
  public:
   Header()
       : file_size_(0),
@@ -38,9 +38,7 @@ class Header {
         max_num_blocks_(0),
         next_key_pos_(0),
         key_buf_size_(0),
-        leaders_(),
-        status_flags_(0),
-        reserved_() {
+        status_flags_(0) {
     for (UInt32 i = 0; i <= MAX_BLOCK_LEVEL; ++i) {
       leaders_[i] = INVALID_LEADER;
     }
@@ -179,5 +177,3 @@ class Header {
 
 }  // namespace dat
 }  // namespace grn
-
-#endif  // GRN_DAT_HPP_HEADER_HPP_
