@@ -1,6 +1,7 @@
 /* -*- c-basic-offset: 2 -*- */
 /*
-  Copyright(C) 2009-2017 Brazil
+  Copyright(C) 2009-2017  Brazil
+  Copyright(C) 2020  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -19,6 +20,7 @@
 #include "grn_geo.h"
 #include "grn_pat.h"
 #include "grn_util.h"
+#include "grn_posting.h"
 
 #include <string.h>
 #include <stdlib.h>
@@ -52,6 +54,7 @@ typedef struct {
   grn_obj bottom_right_point_buffer;
   grn_geo_point *top_left;
   grn_geo_point *bottom_right;
+  grn_geo_point center;
 } in_rectangle_data;
 
 typedef struct {
@@ -814,18 +817,41 @@ grn_selector_geo_in_circle(grn_ctx *ctx, grn_obj *table, grn_obj *index,
                            int nargs, grn_obj **args,
                            grn_obj *res, grn_operator op)
 {
+  const char *tag = "[geo-in-circle]";
+  grn_selector_data *data = grn_selector_data_get(ctx);
   grn_geo_approximate_type type = GRN_GEO_APPROXIMATE_RECTANGLE;
 
   if (!(nargs == 4 || nargs == 5)) {
     ERR(GRN_INVALID_ARGUMENT,
-        "geo_in_circle(): requires 3 or 4 arguments but was <%d> arguments",
+        "%s requires 3 or 4 arguments but was <%d> arguments",
+        tag,
         nargs - 1);
     return ctx->rc;
   }
 
   if (nargs == 5) {
-    if (grn_geo_resolve_approximate_type(ctx, args[4], &type) != GRN_SUCCESS) {
-      return ctx->rc;
+    grn_obj *type_name = NULL;
+    if (args[4]->header.type == GRN_TABLE_HASH_KEY) {
+      grn_obj *options = args[4];
+      grn_rc rc = grn_selector_data_parse_options(ctx,
+                                                  data,
+                                                  options,
+                                                  tag,
+                                                  "type",
+                                                  GRN_PROC_OPTION_VALUE_RAW,
+                                                  &type_name,
+                                                  NULL);
+      if (rc != GRN_SUCCESS) {
+        return rc;
+      }
+    } else {
+      type_name = args[4];
+    }
+    if (type_name) {
+      grn_rc rc = grn_geo_resolve_approximate_type(ctx, type_name, &type);
+      if (rc != GRN_SUCCESS) {
+        return rc;
+      }
     }
   }
 
@@ -840,9 +866,9 @@ grn_selector_geo_in_circle(grn_ctx *ctx, grn_obj *table, grn_obj *index,
 }
 
 static grn_geo_distance_raw_func
-grn_geo_resolve_distance_raw_func (grn_ctx *ctx,
-                                   grn_geo_approximate_type approximate_type,
-                                   grn_id domain)
+grn_geo_resolve_distance_raw_func(grn_ctx *ctx,
+                                  grn_geo_approximate_type approximate_type,
+                                  grn_id domain)
 {
   grn_geo_distance_raw_func distance_raw_func = NULL;
 
@@ -960,6 +986,11 @@ grn_geo_select_in_circle(grn_ctx *ctx, grn_obj *index,
     on_circle.latitude = center->latitude + GRN_GEO_RAD2INT(d / (double)GRN_GEO_RADIUS);
     on_circle.longitude = center->longitude;
     break;
+  case GRN_DB_FLOAT32 :
+    d = GRN_FLOAT32_VALUE(distance);
+    on_circle.latitude = center->latitude + GRN_GEO_RAD2INT(d / (double)GRN_GEO_RADIUS);
+    on_circle.longitude = center->longitude;
+    break;
   case GRN_DB_FLOAT :
     d = GRN_FLOAT_VALUE(distance);
     on_circle.latitude = center->latitude + GRN_GEO_RAD2INT(d / (double)GRN_GEO_RADIUS);
@@ -989,6 +1020,10 @@ grn_geo_select_in_circle(grn_ctx *ctx, grn_obj *index,
     goto exit;
   }
   {
+    grn_selector_data *data = grn_selector_data_get(ctx);
+    const bool use_selector_data =
+      (data && grn_selector_data_have_score_column(ctx, data));
+
     int n_meshes, diff_bit;
     double d_far;
     mesh_entry meshes[87];
@@ -1030,7 +1065,15 @@ grn_geo_select_in_circle(grn_ctx *ctx, grn_obj *index,
           point_distance = distance_raw_func(ctx, &point, center);
           if (point_distance <= d) {
             inspect_tid(ctx, tid, &point, point_distance);
-            grn_ii_at(ctx, (grn_ii *)index, tid, (grn_hash *)res, op);
+            if (use_selector_data) {
+              grn_selector_data_on_token_found(ctx,
+                                               data,
+                                               index,
+                                               tid,
+                                               point_distance);
+            } else {
+              grn_ii_at(ctx, (grn_ii *)index, tid, (grn_hash *)res, op);
+            }
           }
         }
         grn_table_cursor_close(ctx, tc);
@@ -1047,18 +1090,47 @@ grn_selector_geo_in_rectangle(grn_ctx *ctx, grn_obj *table, grn_obj *index,
                               int nargs, grn_obj **args,
                               grn_obj *res, grn_operator op)
 {
-  if (nargs == 4) {
-    grn_obj *top_left_point, *bottom_right_point;
-    top_left_point = args[2];
-    bottom_right_point = args[3];
-    grn_geo_select_in_rectangle(ctx, index,
-                                top_left_point, bottom_right_point,
-                                res, op);
-  } else {
+  const char *tag = "[geo-in-rectangle]";
+  grn_selector_data *data = grn_selector_data_get(ctx);
+
+  if (!(nargs == 4 || nargs == 5)) {
     ERR(GRN_INVALID_ARGUMENT,
-        "geo_in_rectangle(): requires 3 arguments but was <%d> arguments",
+        "%s requires 3 or 4 arguments but was <%d> arguments",
+        tag,
         nargs - 1);
+    return ctx->rc;
   }
+
+  if (nargs == 5) {
+    if (args[4]->header.type != GRN_TABLE_HASH_KEY) {
+      grn_obj inspected;
+      GRN_TEXT_INIT(&inspected, 0);
+      grn_inspect(ctx, &inspected, args[4]);
+      ERR(GRN_INVALID_ARGUMENT,
+          "%s the 4th argument must be options: %.*s",
+          tag,
+          (int)GRN_TEXT_LEN(&inspected),
+          GRN_TEXT_VALUE(&inspected));
+      GRN_OBJ_FIN(ctx, &inspected);
+      return ctx->rc;
+    }
+    grn_obj *options = args[4];
+    grn_rc rc = grn_selector_data_parse_options(ctx,
+                                                data,
+                                                options,
+                                                tag,
+                                                NULL);
+    if (rc != GRN_SUCCESS) {
+      return rc;
+    }
+  }
+
+  grn_obj *top_left_point, *bottom_right_point;
+  top_left_point = args[2];
+  bottom_right_point = args[3];
+  grn_geo_select_in_rectangle(ctx, index,
+                              top_left_point, bottom_right_point,
+                              res, op);
   return ctx->rc;
 }
 
@@ -1151,6 +1223,13 @@ in_rectangle_data_fill(grn_ctx *ctx, grn_obj *index,
     bottom_right_point = &(data->bottom_right_point_buffer);
   }
   data->bottom_right = GRN_GEO_POINT_VALUE_RAW(bottom_right_point);
+
+  data->center.latitude =
+    data->top_left->latitude -
+    (abs(data->top_left->latitude - data->bottom_right->latitude) / 2);
+  data->center.longitude =
+    data->bottom_right->longitude -
+    (abs(data->bottom_right->longitude - data->top_left->longitude) / 2);
 }
 
 static void
@@ -1540,9 +1619,13 @@ grn_geo_cursor_open_in_rectangle(grn_ctx *ctx,
   }
 
   cursor->pat = data.pat;
+  grn_obj_refer(ctx, cursor->pat);
   cursor->index = index;
-  grn_memcpy(&(cursor->top_left), data.top_left, sizeof(grn_geo_point));
-  grn_memcpy(&(cursor->bottom_right), data.bottom_right, sizeof(grn_geo_point));
+  grn_obj_refer(ctx, cursor->index);
+  cursor->top_left = *(data.top_left);
+  cursor->bottom_right = *(data.bottom_right);
+  cursor->center = data.center;
+  cursor->need_distance = false;
   cursor->pat_cursor = NULL;
   cursor->ii_cursor = NULL;
   cursor->offset = offset;
@@ -1590,6 +1673,16 @@ exit :
   GRN_API_RETURN((grn_obj *)cursor);
 }
 
+static void
+grn_geo_cursor_in_rectangle_set_need_distance(grn_ctx *ctx,
+                                              grn_obj *cursor,
+                                              bool need_distance)
+{
+  grn_geo_cursor_in_rectangle *cursor_in_rectangle =
+    (grn_geo_cursor_in_rectangle *)cursor;
+  cursor_in_rectangle->need_distance = need_distance;
+}
+
 static grn_inline grn_bool
 grn_geo_cursor_entry_next_push(grn_ctx *ctx,
                                grn_geo_cursor_in_rectangle *cursor,
@@ -1855,11 +1948,16 @@ grn_geo_cursor_entry_next(grn_ctx *ctx,
   return GRN_TRUE;
 }
 
-typedef grn_bool (*grn_geo_cursor_callback)(grn_ctx *ctx, grn_posting *posting, void *user_data);
+typedef grn_bool (*grn_geo_cursor_callback)(grn_ctx *ctx,
+                                            grn_posting *posting,
+                                            double distance,
+                                            void *user_data);
 
 static void
-grn_geo_cursor_each(grn_ctx *ctx, grn_obj *geo_cursor,
-                    grn_geo_cursor_callback callback, void *user_data)
+grn_geo_cursor_each(grn_ctx *ctx,
+                    grn_obj *geo_cursor,
+                    grn_geo_cursor_callback callback,
+                    void *user_data)
 {
   grn_geo_cursor_in_rectangle *cursor;
   grn_obj *pat;
@@ -1931,7 +2029,13 @@ grn_geo_cursor_each(grn_ctx *ctx, grn_obj *geo_cursor,
       while ((posting = grn_ii_cursor_next(ctx, ii_cursor))) {
         if (cursor->offset == 0) {
           grn_bool keep_each;
-          keep_each = callback(ctx, posting, user_data);
+          double distance = 0.0;
+          if (cursor->need_distance) {
+            distance = grn_geo_distance_rectangle_raw(ctx,
+                                                      &(cursor->center),
+                                                      &(cursor->current));
+          }
+          keep_each = callback(ctx, posting, distance, user_data);
           if (cursor->rest > 0) {
             if (--(cursor->rest) == 0) {
               keep_each = GRN_FALSE;
@@ -1953,7 +2057,9 @@ grn_geo_cursor_each(grn_ctx *ctx, grn_obj *geo_cursor,
 }
 
 static grn_bool
-grn_geo_cursor_next_callback(grn_ctx *ctx, grn_posting *posting,
+grn_geo_cursor_next_callback(grn_ctx *ctx,
+                             grn_posting *posting,
+                             double distance,
                              void *user_data)
 {
   grn_posting **return_posting = user_data;
@@ -1966,7 +2072,7 @@ grn_geo_cursor_next(grn_ctx *ctx, grn_obj *geo_cursor)
 {
   grn_posting *posting = NULL;
   grn_geo_cursor_each(ctx, geo_cursor, grn_geo_cursor_next_callback, &posting);
-  return (grn_posting *)posting;
+  return posting;
 }
 
 grn_rc
@@ -1987,19 +2093,30 @@ grn_geo_cursor_close(grn_ctx *ctx, grn_obj *geo_cursor)
 }
 
 typedef struct {
+  grn_selector_data *selector_data;
+  bool use_distance;
   grn_hash *res;
   grn_operator op;
 } grn_geo_select_in_rectangle_data;
 
 static grn_bool
-grn_geo_select_in_rectangle_callback(grn_ctx *ctx, grn_posting *posting,
+grn_geo_select_in_rectangle_callback(grn_ctx *ctx,
+                                     grn_posting *posting,
+                                     double distance,
                                      void *user_data)
 {
   grn_geo_select_in_rectangle_data *data = user_data;
-  grn_posting add_posting = *posting;
-  add_posting.weight += 1;
-  grn_ii_posting_add(ctx, &add_posting, data->res, data->op);
-  return GRN_TRUE;
+  grn_posting_internal add_posting = *((grn_posting_internal *)posting);
+  if (data->use_distance) {
+    add_posting.weight_float = (add_posting.weight_float + 1.0) * distance;
+  } else {
+    add_posting.weight_float += 1.0;
+  }
+  grn_ii_posting_add_float(ctx,
+                           (grn_posting *)(&add_posting),
+                           data->res,
+                           data->op);
+  return ctx->rc == GRN_SUCCESS;
 }
 
 grn_rc
@@ -2010,14 +2127,23 @@ grn_geo_select_in_rectangle(grn_ctx *ctx, grn_obj *index,
 {
   grn_obj *cursor;
 
+  grn_geo_select_in_rectangle_data data;
+  data.selector_data = grn_selector_data_get(ctx);
+  data.use_distance =
+    (data.selector_data &&
+     grn_selector_data_have_score_column(ctx, data.selector_data));
+  data.res = (grn_hash *)res;
+  data.op = op;
   cursor = grn_geo_cursor_open_in_rectangle(ctx, index,
                                             top_left_point, bottom_right_point,
                                             0, -1);
   if (cursor) {
-    grn_geo_select_in_rectangle_data data;
-    data.res = (grn_hash *)res;
-    data.op = op;
-    grn_geo_cursor_each(ctx, cursor, grn_geo_select_in_rectangle_callback,
+    grn_geo_cursor_in_rectangle_set_need_distance(ctx,
+                                                  cursor,
+                                                  data.use_distance);
+    grn_geo_cursor_each(ctx,
+                        cursor,
+                        grn_geo_select_in_rectangle_callback,
                         &data);
     grn_obj_unlink(ctx, cursor);
     grn_ii_resolve_sel_and(ctx, (grn_hash *)res, op);
@@ -2189,6 +2315,9 @@ grn_geo_in_circle(grn_ctx *ctx, grn_obj *point, grn_obj *center,
     case GRN_DB_UINT64 :
       r = d <= GRN_UINT64_VALUE(radius_or_point);
       break;
+    case GRN_DB_FLOAT32 :
+      r = d <= GRN_FLOAT32_VALUE(radius_or_point);
+      break;
     case GRN_DB_FLOAT :
       r = d <= GRN_FLOAT_VALUE(radius_or_point);
       break;
