@@ -1,7 +1,7 @@
 /* -*- c-basic-offset: 2 -*- */
 /*
-  Copyright(C) 2009-2018 Brazil
-  Copyright(C) 2018 Kouhei Sutou <kou@clear-code.com>
+  Copyright(C) 2009-2018  Brazil
+  Copyright(C) 2018-2020  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -29,12 +29,7 @@
 #  define __MINGW_MSVC_COMPAT_WARNINGS
 # endif /* __GNUC__ */
 
-# ifdef __GNUC__
-#  include <w32api.h>
-#  define GRN_MINIMUM_WINDOWS_VERSION WindowsVista
-# else /* __GNUC__ */
-#  define GRN_MINIMUM_WINDOWS_VERSION 0x0600 /* Vista */
-# endif /* __GNUC__ */
+# define GRN_MINIMUM_WINDOWS_VERSION _WIN32_WINNT_WIN8
 
 # ifdef WINVER
 #  undef WINVER
@@ -44,10 +39,6 @@
 #  undef _WIN32_WINNT
 # endif /* _WIN32_WINNT */
 # define _WIN32_WINNT GRN_MINIMUM_WINDOWS_VERSION
-# ifdef NTDDI_VERSION
-#  undef NTDDI_VERSION
-# endif /* NTDDI_VERSION */
-# define NTDDI_VERSION GRN_MINIMUM_WINDOWS_VERSION
 
 # ifdef WIN32_LEAN_AND_MEAN
 #  undef WIN32_LEAN_AND_MEAN
@@ -102,7 +93,9 @@
 #endif /* WIN32 */
 
 #ifdef WIN32
-# define NOMINMAX
+# ifndef NOMINMAX
+#  define NOMINMAX
+# endif
 # include <basetsd.h>
 # include <process.h>
 # include <winsock2.h>
@@ -136,6 +129,7 @@ typedef SOCKET grn_sock;
 
 # ifndef __GNUC__
 #  include <intrin.h>
+#  pragma intrinsic(_BitScanReverse)
 #  pragma intrinsic(_InterlockedExchangeAdd)
 #  include <sys/timeb.h>
 #  include <errno.h>
@@ -359,13 +353,20 @@ typedef int grn_cond;
 # define GRN_FMT_INT64D "I64d"
 # define GRN_FMT_INT64U "I64u"
 # define GRN_FMT_INT64X "I64x"
-#elif defined(HAVE_INTTYPES_H)
+#elif defined(HAVE_INTTYPES_H) && !defined(__cplusplus)
 # include <inttypes.h>
 # define GRN_FMT_INT32D PRId32
 # define GRN_FMT_INT32U PRIu32
 # define GRN_FMT_INT64D PRId64
 # define GRN_FMT_INT64U PRIu64
 # define GRN_FMT_INT64X PRIx64
+#elif defined(__cplusplus) && __cplusplus >= 201103L
+# include <cinttypes>
+# define GRN_FMT_INT32D PRId32
+# define GRN_FMT_INT32U PRIu32
+# define GRN_FMT_INT64D PRId64
+# define GRN_FMT_INT64U PRIu64
+# define GRN_FMT_INT64X PRIx64
 #else /* WIN32 */
 # define GRN_FMT_INT32D "d"
 # define GRN_FMT_INT32U "u"
@@ -489,7 +490,11 @@ typedef int grn_cond;
 # endif /* ATOMIC 64BIT SET */
 
 /* todo */
-# define GRN_BIT_SCAN_REV(v,r)  for (r = 31; r && !((1 << r) & v); r--)
+# define GRN_BIT_SCAN_REV(v,r)  do {            \
+    if (!BitScanReverse(&r, v)) {               \
+      r = 0;                                    \
+    }                                           \
+  } while (0)
 # define GRN_BIT_SCAN_REV0(v,r) GRN_BIT_SCAN_REV(v,r)
 
 #else /* __GNUC__ */
@@ -682,6 +687,14 @@ grn_str_greater(const uint8_t *ap, uint32_t as, const uint8_t *bp, uint32_t bs)
 # define GRN_SUPPORT_REGEXP
 #endif /* GRN_WITH_ONIGMO */
 
+#ifdef __GNUC__
+# define GRN_LIKELY(expr) (__builtin_expect(!!(expr), 1))
+# define GRN_UNLIKELY(expr) (__builtin_expect(!!(expr), 0))
+#else
+# define GRN_LIKELY(expr) expr
+# define GRN_UNLIKELY(expr) expr
+#endif
+
 GRN_API void grn_sleep(uint32_t seconds);
 GRN_API void grn_nanosleep(uint64_t nanoseconds);
 
