@@ -1,7 +1,7 @@
 /* -*- c-basic-offset: 2 -*- */
 /*
-  Copyright(C) 2009-2018 Brazil
-  Copyright(C) 2018-2019 Kouhei Sutou <kou@clear-code.com>
+  Copyright(C) 2009-2018  Brazil
+  Copyright(C) 2018-2021  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -24,10 +24,7 @@
 #include "grn_options.h"
 #include "grn_msgpack.h"
 #include "grn_load.h"
-
-#ifdef GRN_WITH_APACHE_ARROW
-# include "grn_arrow.h"
-#endif
+#include "grn_arrow.h"
 
 #ifdef GRN_WITH_MRUBY
 # include <mruby.h>
@@ -124,13 +121,12 @@ struct _grn_ctx_impl {
   void *current_request_timer_id;
   void *parser;
   grn_timeval tv;
+  grn_selector_data *current_selector_data;
 
   /* loader portion */
   grn_edge *edge;
   grn_loader loader;
-#ifdef GRN_WITH_APACHE_ARROW
   grn_arrow_stream_loader *arrow_stream_loader;
-#endif
 
   /* plugin portion */
   const char *plugin_path;
@@ -153,6 +149,7 @@ struct _grn_ctx_impl {
 #ifdef GRN_WITH_MESSAGE_PACK
     msgpack_packer msgpacker;
 #endif
+    grn_arrow_stream_writer *arrow_stream_writer;
   } output;
 
   struct {
@@ -193,6 +190,8 @@ struct _grn_ctx_impl {
     grn_obj stack;
     grn_obj *current;
   } temporary_open_spaces;
+
+  grn_hash *variables;
 };
 
 void grn_ctx_impl_columns_cache_delete(grn_ctx *ctx, grn_id table_id);
