@@ -21,6 +21,7 @@
 
 #include "grn.h"
 #include "grn_ctx.h"
+#include "grn_ii_select_cursor.h"
 #include "grn_table_module.h"
 
 #ifdef __cplusplus
@@ -357,7 +358,27 @@ grn_rc
 grn_hash_add_table_cursor(grn_ctx *ctx,
                           grn_hash *hash,
                           grn_table_cursor *cursor,
-                          double score);
+                          double score,
+                          grn_operator op);
+grn_rc
+grn_hash_add_index_cursor(grn_ctx *ctx,
+                          grn_hash *hash,
+                          grn_obj *cursor,
+                          double additional_score,
+                          double weight,
+                          grn_operator op);
+grn_rc
+grn_hash_add_ii_cursor(grn_ctx *ctx,
+                       grn_hash *hash,
+                       grn_ii_cursor *cursor,
+                       double additional_score,
+                       double weight,
+                       grn_operator op);
+grn_rc
+grn_hash_add_ii_select_cursor(grn_ctx *ctx,
+                              grn_hash *hash,
+                              grn_ii_select_cursor *cursor,
+                              grn_operator op);
 
 int grn_hash_get_key_value(grn_ctx *ctx, grn_hash *hash, grn_id id,
                            void *keybuf, int bufsize, void *valuebuf);
