@@ -56,7 +56,6 @@ typedef struct {
   uint32_t offset;
   uint32_t size;
   uint32_t nseg;
-  off_t pos;
   void *addr;
   uint32_t diff;
   int32_t cached;
@@ -318,19 +317,19 @@ static grn_inline void *
 grn_io_array_at(grn_ctx *ctx,
                 grn_io *io,
                 uint32_t array,
-                off_t offset,
+                int64_t offset,
                 int *flags)
 {
   grn_io_array_info *ainfo = &(io->ainfo[array]);
   uint32_t lseg = offset >> ainfo->w_of_elm_in_a_segment;
   if (lseg >= ainfo->max_n_segments) {
     ERR(GRN_NO_MEMORY_AVAILABLE,
-        "[io][array][at] too large offset: %" GRN_FMT_INT64U
-        ": max=%" GRN_FMT_INT64U
+        "[io][array][at] too large offset: %" GRN_FMT_INT64D
+        ": max=%" GRN_FMT_INT64D
         ": nth=%u"
         ": path=<%s>",
-        (uint64_t)offset,
-        (uint64_t)((ainfo->max_n_segments << ainfo->w_of_elm_in_a_segment) - 1),
+        offset,
+        (int64_t)((ainfo->max_n_segments << ainfo->w_of_elm_in_a_segment) - 1),
         array,
         io->path);
     return NULL;
