@@ -1,6 +1,7 @@
 /* -*- c-basic-offset: 2 -*- */
 /*
   Copyright(C) 2012-2016 Brazil
+  Copyright(C) 2018 Kouhei Sutou <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -21,6 +22,8 @@
 #include "grn.h"
 #include "grn_ctx.h"
 #include "grn_db.h"
+#include "grn_nfkc.h"
+#include "grn_string.h"
 
 #ifdef __cplusplus
 extern "C" {
@@ -35,6 +38,10 @@ grn_rc grn_normalizer_normalize(grn_ctx *ctx,
                                 grn_obj *normalizer,
                                 grn_obj *string);
 
+grn_rc grn_nfkc_normalize(grn_ctx *ctx,
+                          grn_obj *string,
+                          grn_nfkc_normalize_options *options);
+
 grn_rc grn_db_init_builtin_normalizers(grn_ctx *ctx);
 
 #ifdef __cplusplus
