@@ -20,14 +20,14 @@
 #pragma once
 
 #include "grn.h"
-#include "grn_ctx.h"
-#include "grn_store.h"
-#include "grn_ctx_impl.h"
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
+extern uint32_t grn_output_auto_flush_interval;
+void grn_output_init_from_env(void);
+
 GRN_API void grn_output_result_set_open(grn_ctx *ctx,
                                         grn_obj *outbuf,
                                         grn_content_type output_type,
