@@ -1,7 +1,7 @@
 /* -*- c-basic-offset: 2 -*- */
 /*
-  Copyright(C) 2009-2018 Brazil
-  Copyright(C) 2019 Sutou Kouhei <kou@clear-code.com>
+  Copyright(C) 2009-2018  Brazil
+  Copyright(C) 2019-2021  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -28,10 +28,6 @@ extern "C" {
 #define GRN_SELECT_DEFAULT_LIMIT           10
 #define GRN_SELECT_DEFAULT_OUTPUT_COLUMNS  "_id, _key, *"
 
-#define GRN_SELECT_INTERNAL_VAR_CONDITION     "$condition"
-#define GRN_SELECT_INTERNAL_VAR_CONDITION_LEN           \
-  (sizeof(GRN_SELECT_INTERNAL_VAR_CONDITION) - 1)
-
 void grn_proc_init_from_env(void);
 
 GRN_VAR const char *grn_document_root;
@@ -65,11 +61,14 @@ void grn_proc_init_object_inspect(grn_ctx *ctx);
 void grn_proc_init_object_list(grn_ctx *ctx);
 void grn_proc_init_object_remove(grn_ctx *ctx);
 void grn_proc_init_object_set_visibility(grn_ctx *ctx);
+void grn_proc_init_query(grn_ctx *ctx);
 void grn_proc_init_query_expand(grn_ctx *ctx);
 void grn_proc_init_query_log_flags_get(grn_ctx *ctx);
 void grn_proc_init_query_log_flags_set(grn_ctx *ctx);
 void grn_proc_init_query_log_flags_add(grn_ctx *ctx);
 void grn_proc_init_query_log_flags_remove(grn_ctx *ctx);
+void grn_proc_init_reference_acquire(grn_ctx *ctx);
+void grn_proc_init_reference_release(grn_ctx *ctx);
 void grn_proc_init_schema(grn_ctx *ctx);
 void grn_proc_init_select(grn_ctx *ctx);
 void grn_proc_init_snippet(grn_ctx *ctx);
@@ -110,6 +109,16 @@ grn_operator grn_proc_get_value_operator(grn_ctx *ctx,
                                          grn_obj *value,
                                          grn_operator default_operator,
                                          const char *context);
+grn_tokenize_mode
+grn_proc_get_value_tokenize_mode(grn_ctx *ctx,
+                                 grn_obj *value,
+                                 grn_tokenize_mode default_mode,
+                                 const char *context);
+uint32_t
+grn_proc_get_value_token_cursor_flags(grn_ctx *ctx,
+                                      grn_obj *value,
+                                      uint32_t default_flags,
+                                      const char *context);
 
 void grn_proc_output_object_name(grn_ctx *ctx, grn_obj *obj);
 void grn_proc_output_object_id_name(grn_ctx *ctx, grn_id id);
@@ -172,6 +181,26 @@ grn_bool grn_proc_text_include_special_character(grn_ctx *ctx,
                                                  const char *text,
                                                  size_t size);
 
+typedef struct {
+  grn_obj *found;
+  grn_obj *table;
+  grn_obj *records;
+} grn_proc_selector_to_function_data;
+
+bool
+grn_proc_selector_to_function_data_init(grn_ctx *ctx,
+                                        grn_proc_selector_to_function_data *data,
+                                        grn_user_data *user_data);
+void
+grn_proc_selector_to_function_data_selected(
+  grn_ctx *ctx,
+  grn_proc_selector_to_function_data *data);
+void
+grn_proc_selector_to_function_data_fin(grn_ctx *ctx,
+                                       grn_proc_selector_to_function_data *data);
+
+
+
 #ifdef __cplusplus
 }
 #endif
