@@ -63,6 +63,7 @@
 #endif /* WIN32 */
 
 typedef struct _grn_io_fileinfo {
+  char *path;
 #ifdef WIN32
   HANDLE fh;
   HANDLE fmo;
@@ -89,7 +90,7 @@ grn_inline static void *grn_mmap(grn_ctx *ctx,
                                  grn_io *io,
                                  HANDLE *fmo,
                                  fileinfo *fi,
-                                 off_t offset,
+                                 int64_t offset,
                                  size_t length,
                                  const char *file,
                                  int line,
@@ -107,7 +108,7 @@ grn_inline static void *grn_mmap(grn_ctx *ctx,
                                  grn_ctx *owner_ctx,
                                  grn_io *io,
                                  fileinfo *fi,
-                                 off_t offset,
+                                 int64_t offset,
                                  size_t length,
                                  const char *file,
                                  int line,
@@ -518,14 +519,6 @@ grn_io_segment_alloc(grn_ctx *ctx, grn_io *io, grn_io_array_info *ai,
   }
 }
 
-void *
-grn_io_array_at(grn_ctx *ctx, grn_io *io, uint32_t array, off_t offset, int *flags)
-{
-  void *res;
-  GRN_IO_ARRAY_AT(io,array,offset,flags,res);
-  return res;
-}
-
 uint32_t
 grn_io_detect_type(grn_ctx *ctx, const char *path)
 {
@@ -1623,7 +1616,7 @@ grn_fileinfo_open_v1(grn_ctx *ctx, fileinfo *fi, const char *path, int flags)
 
 grn_inline static void *
 grn_mmap_v1(grn_ctx *ctx, grn_ctx *owner_ctx, HANDLE *fmo, fileinfo *fi,
-            off_t offset, size_t length)
+            int64_t offset, size_t length)
 {
   void *res;
   if (!fi) {
@@ -1638,6 +1631,7 @@ grn_mmap_v1(grn_ctx *ctx, grn_ctx *owner_ctx, HANDLE *fmo, fileinfo *fi,
   }
   /* CRITICAL_SECTION_ENTER(fi->cs); */
   /* try to create fmo */
+  /* TODO: Add support for 32bit over offset by using dwMaximumSizeHigh. */
   *fmo = CreateFileMapping(fi->fh, NULL, PAGE_READWRITE, 0, offset + length, NULL);
   if (!*fmo) {
     SERR("CreateFileMapping(%lu + %" GRN_FMT_SIZE ") failed "
@@ -1731,11 +1725,15 @@ grn_fileinfo_open_v0(grn_ctx *ctx, fileinfo *fi, const char *path, int flags)
   }
   CloseHandle(fi->fh);
   SERR("OpenFileMapping");
+  if (ctx->rc != GRN_SUCCESS && fi->path) {
+    free(fi->path);
+    fi->path = NULL;
+  }
   return ctx->rc;
 }
 
 grn_inline static void *
-grn_mmap_v0(grn_ctx *ctx, grn_ctx *owner_ctx, fileinfo *fi, off_t offset,
+grn_mmap_v0(grn_ctx *ctx, grn_ctx *owner_ctx, fileinfo *fi, int64_t offset,
             size_t length)
 {
   void *res;
@@ -1756,6 +1754,7 @@ grn_mmap_v0(grn_ctx *ctx, grn_ctx *owner_ctx, fileinfo *fi, off_t offset,
     filesize = tail;
   }
   */
+  /* TODO: Add support for 32bit over offset by using dwFileOffsetHigh. */
   res = MapViewOfFile(fi->fmo, FILE_MAP_WRITE, 0, (DWORD)offset, (SIZE_T)length);
   if (!res) {
     MERR("MapViewOfFile failed: <%" GRN_FMT_SIZE ">: %s",
@@ -1797,6 +1796,8 @@ grn_fileinfo_open_common(grn_ctx *ctx, fileinfo *fi, const char *path, int flags
 {
   bool success = true;
 
+  fi->path = grn_strdup_raw(path);
+
   /* may be wrong if flags is just only O_RDWR */
   if ((flags & O_CREAT)) {
     grn_bool exist = GRN_FALSE;
@@ -1912,6 +1913,10 @@ grn_fileinfo_open_common(grn_ctx *ctx, fileinfo *fi, const char *path, int flags
           "[io][open] open existing file: <%s>", path);
 
 exit :
+  if (!success && fi->path) {
+    free(fi->path);
+    fi->path = NULL;
+  }
   return success;
 }
 
@@ -1972,7 +1977,7 @@ grn_mmap(grn_ctx *ctx,
          grn_io *io,
          HANDLE *fmo,
          fileinfo *fi,
-         off_t offset,
+         int64_t offset,
          size_t length,
          const char *file,
          int line,
@@ -1980,13 +1985,13 @@ grn_mmap(grn_ctx *ctx,
 {
   if (grn_fail_malloc_should_fail(length, file, line, func)) {
     MERR("[alloc][fail][mmap] <%d>: <%" GRN_FMT_SIZE ">: <%s>: "
-         "<%p:%" GRN_FMT_INT64U ":%" GRN_FMT_SIZE ">: "
+         "<%p:%" GRN_FMT_INT64D ":%" GRN_FMT_SIZE ">: "
          "%s:%d: %s",
          grn_alloc_count(),
          mmap_size,
          io ? (io->path[0] ? io->path : "(memory)") : "(null)",
          fi ? fi->fh : NULL,
-         (uint64_t)offset,
+         offset,
          length,
          file,
          line,
@@ -2029,21 +2034,16 @@ grn_fileinfo_close(grn_ctx *ctx, fileinfo *fi)
   }
   if (fi->fh != INVALID_HANDLE_VALUE) {
     if (grn_logger_pass(ctx, GRN_LOG_DEBUG)) {
-      TCHAR path[MAX_PATH];
-      DWORD path_length_including_null =
-        GetFinalPathNameByHandleA(fi->fh,
-                                  path,
-                                  MAX_PATH,
-                                  0);
-      if (path_length_including_null != 0 &&
-          path_length_including_null <= MAX_PATH) {
-        GRN_LOG(ctx, GRN_LOG_DEBUG, "[io][close] <%s>", path);
-      }
+      GRN_LOG(ctx, GRN_LOG_DEBUG, "[io][close] <%s>", fi->path);
     }
     CloseHandle(fi->fh);
     CRITICAL_SECTION_FIN(fi->cs);
     fi->fh = INVALID_HANDLE_VALUE;
   }
+  if (fi->path) {
+    free(fi->path);
+    fi->path = NULL;
+  }
   return GRN_SUCCESS;
 }
 
@@ -2051,6 +2051,7 @@ grn_inline static void
 grn_fileinfo_init(fileinfo *fis, int nfis)
 {
   for (; nfis--; fis++) {
+    fis->path = NULL;
     fis->fh = INVALID_HANDLE_VALUE;
     fis->fmo = NULL;
   }
@@ -2154,15 +2155,21 @@ grn_inline static grn_rc
 grn_fileinfo_open(grn_ctx *ctx, fileinfo *fi, const char *path, int flags)
 {
   struct stat st;
+  fi->path = grn_strdup_raw(path);
   grn_open(fi->fd, path, flags);
   if (fi->fd == -1) {
     ERRNO_ERR("failed to open file info path: <%s>",
               path);
+    free(fi->path);
+    fi->path = NULL;
     return ctx->rc;
   }
   if (fstat(fi->fd, &st) == -1) {
     ERRNO_ERR("failed to stat file info path: <%s>",
               path);
+    grn_close(fi->fd);
+    free(fi->path);
+    fi->path = NULL;
     return ctx->rc;
   }
   fi->dev = st.st_dev;
@@ -2174,7 +2181,10 @@ grn_fileinfo_open(grn_ctx *ctx, fileinfo *fi, const char *path, int flags)
 grn_inline static void
 grn_fileinfo_init(fileinfo *fis, int nfis)
 {
-  for (; nfis--; fis++) { fis->fd = -1; }
+  for (; nfis--; fis++) {
+    fis->path = NULL;
+    fis->fd = -1;
+  }
 }
 
 grn_inline static int
@@ -2187,28 +2197,19 @@ grn_inline static grn_rc
 grn_fileinfo_close(grn_ctx *ctx, fileinfo *fi)
 {
   if (fi->fd != -1) {
-    char path[PATH_MAX];
-    path[0] = '\0';
-    if (grn_logger_pass(ctx, GRN_LOG_DUMP)) {
-      /* TODO: This works only on Linux. We need to use more portable way. */
-      char proc_path[PATH_MAX];
-      grn_snprintf(proc_path, PATH_MAX, PATH_MAX, "/proc/self/fd/%d", fi->fd);
-      ssize_t path_size = readlink(proc_path, path, PATH_MAX);
-      if (path_size == -1) {
-        errno = 0;
-      } else {
-        path[path_size] = '\0';
-      }
-    }
     if (grn_close(fi->fd) == -1) {
       SERR("close");
       return ctx->rc;
     }
     fi->fd = -1;
-    if (path[0]) {
-      GRN_LOG(ctx, GRN_LOG_DUMP, "[io][close] <%s>", path);
+    if (fi->path) {
+      GRN_LOG(ctx, GRN_LOG_DUMP, "[io][close] <%s>", fi->path);
     }
   }
+  if (fi->path) {
+    free(fi->path);
+    fi->path = NULL;
+  }
   return GRN_SUCCESS;
 }
 
@@ -2223,7 +2224,7 @@ grn_mmap(grn_ctx *ctx,
          grn_ctx *owner_ctx,
          grn_io *io,
          fileinfo *fi,
-         off_t offset,
+         int64_t offset,
          size_t length,
          const char *file,
          int line,
@@ -2231,13 +2232,13 @@ grn_mmap(grn_ctx *ctx,
 {
   if (grn_fail_malloc_should_fail(length, file, line, func)) {
     MERR("[alloc][fail][mmap] <%d>: <%" GRN_FMT_SIZE ">: <%s>: "
-         "<%d:%" GRN_FMT_INT64U ":%" GRN_FMT_SIZE ">: "
+         "<%d:%" GRN_FMT_INT64D ":%" GRN_FMT_SIZE ">: "
          "%s:%d: %s",
          grn_alloc_count(),
          mmap_size,
          io ? (io->path[0] ? io->path : "(memory)") : "(null)",
          fi ? fi->fd : 0,
-         (uint64_t)offset,
+         offset,
          length,
          file,
          line,
@@ -2248,7 +2249,7 @@ grn_mmap(grn_ctx *ctx,
     int fd, flags;
     if (fi) {
       struct stat s;
-      off_t tail = offset + length;
+      int64_t tail = offset + length;
       fd = fi->fd;
       if ((fstat(fd, &s) == -1) || (s.st_size < tail && ftruncate(fd, tail) == -1)) {
         SERR("fstat");
@@ -2261,9 +2262,12 @@ grn_mmap(grn_ctx *ctx,
     }
     res = mmap(NULL, length, PROT_READ|PROT_WRITE, flags, fd, offset);
     if (MAP_FAILED == res) {
-      MERR("mmap(%" GRN_FMT_LLU ",%d,%" GRN_FMT_LLD ")=%s <%" GRN_FMT_LLU ">",
-           (unsigned long long int)length, fd, (long long int)offset,
-           strerror(errno), (unsigned long long int)mmap_size);
+      MERR("mmap(%" GRN_FMT_SIZE ",%d,%" GRN_FMT_INT64D ")=%s <%" GRN_FMT_SIZE ">",
+           length,
+           fd,
+           offset,
+           strerror(errno),
+           mmap_size);
       return NULL;
     }
     mmap_size += length;
@@ -2324,8 +2328,9 @@ grn_pread(grn_ctx *ctx, fileinfo *fi, void *buf, size_t count, off_t offset)
     } else {
       /* todo : should retry ? */
       ERR(GRN_INPUT_OUTPUT_ERROR,
-          "pread returned %" GRN_FMT_LLD " != %" GRN_FMT_LLU,
-          (long long int)r, (unsigned long long int)count);
+          "pread returned %" GRN_FMT_SSIZE " != %" GRN_FMT_SIZE,
+          r,
+          count);
     }
     return ctx->rc;
   }
@@ -2342,8 +2347,9 @@ grn_pwrite(grn_ctx *ctx, fileinfo *fi, void *buf, size_t count, off_t offset)
     } else {
       /* todo : should retry ? */
       ERR(GRN_INPUT_OUTPUT_ERROR,
-          "pwrite returned %" GRN_FMT_LLD " != %" GRN_FMT_LLU,
-          (long long int)r, (unsigned long long int)count);
+          "pwrite returned %" GRN_FMT_SSIZE " != %" GRN_FMT_SIZE,
+          r,
+          count);
     }
     return ctx->rc;
   }
