@@ -138,7 +138,7 @@ grn_log_flags_parse(const char *string,
   }
 
   if (string_size < 0) {
-    string_size = strlen(string);
+    string_size = (int)strlen(string);
   }
 
   string_end = string + string_size;
@@ -524,25 +524,22 @@ grn_logger_putv(grn_ctx *ctx,
     } else {
       mbuf[0] = '\0';
     }
-    if (current_logger.flags & GRN_LOG_LOCATION) {
-      grn_snprintf(lbuf, LBUFSIZE, LBUFSIZE,
-                   "%d %s:%d %s()", grn_getpid(), file, line, func);
-    } else {
+    {
       char *lbuf_current = lbuf;
       size_t lbuf_rest_size = LBUFSIZE;
       lbuf[0] = '\0';
       lbuf_current = lbuf;
-      if (current_logger.flags & GRN_LOG_PID) {
-        size_t lbuf_size;
+      if ((current_logger.flags & GRN_LOG_PID) ||
+          /* For backward compatibility. GRN_LOG_LOCATION implies GRN_LOG_PID. */
+          (current_logger.flags & GRN_LOG_LOCATION)) {
         grn_snprintf(lbuf_current, lbuf_rest_size, lbuf_rest_size,
                      "%d", grn_getpid());
-        lbuf_size = strlen(lbuf_current);
+        const size_t lbuf_size = strlen(lbuf_current);
         lbuf_current += lbuf_size;
         lbuf_rest_size -= lbuf_size;
       }
       if (current_logger.flags & GRN_LOG_THREAD_ID) {
         const char *prefix = "";
-        size_t lbuf_size;
         if (lbuf_current != lbuf) {
           prefix = "|";
         }
@@ -553,13 +550,48 @@ grn_logger_putv(grn_ctx *ctx,
         grn_snprintf(lbuf_current, lbuf_rest_size, lbuf_rest_size,
                      "%s%08ld", prefix, GetCurrentThreadId());
 #endif /* HAVE_PTHREAD_H */
-        lbuf_size = strlen(lbuf_current);
+        const size_t lbuf_size = strlen(lbuf_current);
+        lbuf_current += lbuf_size;
+        lbuf_rest_size -= lbuf_size;
+      }
+      if (current_logger.flags & GRN_LOG_LOCATION) {
+        const char *prefix = "";
+        if (lbuf_current != lbuf) {
+          prefix = " ";
+        }
+        grn_snprintf(lbuf_current, lbuf_rest_size, lbuf_rest_size,
+                     "%s%s:%d %s()", prefix, file, line, func);
+        const size_t lbuf_size = strlen(lbuf_current);
         lbuf_current += lbuf_size;
         lbuf_rest_size -= lbuf_size;
       }
     }
-    current_logger.log(ctx, level, tbuf, "", mbuf, lbuf,
-                       current_logger.user_data);
+
+    if (mbuf[0] == '\0') {
+      current_logger.log(ctx, level, tbuf, "", mbuf, lbuf,
+                         current_logger.user_data);
+    } else {
+      const char *mbuf_line_start = mbuf;
+      const char *mbuf_end = mbuf + strlen(mbuf);
+      int mbuf_char_length = 0;
+      for (char *mbuf_current = mbuf;
+           mbuf_current < mbuf_end;
+           mbuf_current += mbuf_char_length) {
+        mbuf_char_length = grn_charlen(ctx, mbuf_current, mbuf_end);
+        if (mbuf_char_length == 0) {
+          break;
+        } else if (mbuf_char_length == 1 && mbuf_current[0] == '\n') {
+          mbuf_current[0] = '\0';
+          current_logger.log(ctx, level, tbuf, "", mbuf_line_start, lbuf,
+                             current_logger.user_data);
+          mbuf_line_start = mbuf_current + 1;
+        }
+      }
+      if (mbuf_line_start < mbuf_end) {
+        current_logger.log(ctx, level, tbuf, "", mbuf_line_start, lbuf,
+                           current_logger.user_data);
+      }
+    }
   }
 }
 
@@ -609,7 +641,7 @@ grn_query_log_flags_parse(const char *string,
   }
 
   if (string_size < 0) {
-    string_size = strlen(string);
+    string_size = (int)strlen(string);
   }
 
   string_end = string + string_size;
