@@ -33,40 +33,6 @@
 #include "mrb_converter.h"
 #include "mrb_options.h"
 
-static mrb_value
-mrb_grn_table_array_reference(mrb_state *mrb, mrb_value self)
-{
-  grn_ctx *ctx = (grn_ctx *)mrb->ud;
-  grn_obj *table;
-  grn_id key_domain_id;
-  mrb_value mrb_key;
-  grn_id record_id;
-  grn_mrb_value_to_raw_data_buffer buffer;
-  void *key;
-  unsigned int key_size;
-
-  mrb_get_args(mrb, "o", &mrb_key);
-
-  table = DATA_PTR(self);
-  if (table->header.type == GRN_DB) {
-    key_domain_id = GRN_DB_SHORT_TEXT;
-  } else {
-    key_domain_id = table->header.domain;
-  }
-
-  grn_mrb_value_to_raw_data_buffer_init(mrb, &buffer);
-  grn_mrb_value_to_raw_data(mrb, "key", mrb_key, key_domain_id,
-                            &buffer, &key, &key_size);
-  record_id = grn_table_get(ctx, table, key, key_size);
-  grn_mrb_value_to_raw_data_buffer_fin(mrb, &buffer);
-
-  if (record_id == GRN_ID_NIL) {
-    return mrb_nil_value();
-  } else {
-    return mrb_fixnum_value(record_id);
-  }
-}
-
 static mrb_value
 mrb_grn_table_is_id(mrb_state *mrb, mrb_value self)
 {
@@ -147,7 +113,7 @@ mrb_grn_table_get_column_ids(mrb_state *mrb, mrb_value self)
   {
     grn_id *key;
     GRN_HASH_EACH(ctx, columns, id, &key, NULL, NULL, {
-      mrb_ary_push(mrb, mrb_column_ids, mrb_fixnum_value(*key));
+        mrb_ary_push(mrb, mrb_column_ids, mrb_int_value(mrb, *key));
     });
   }
   grn_hash_close(ctx, columns);
@@ -214,7 +180,7 @@ mrb_grn_table_get_size(mrb_state *mrb, mrb_value self)
   size = grn_table_size(ctx, DATA_PTR(self));
   grn_mrb_ctx_check(mrb);
 
-  return mrb_fixnum_value(size);
+  return mrb_int_value(mrb, size);
 }
 
 static mrb_value
@@ -256,7 +222,7 @@ mrb_grn_table_select(mrb_state *mrb, mrb_value self)
 
     mrb_operator = grn_mrb_options_get_lit(mrb, mrb_options, "operator");
     if (!mrb_nil_p(mrb_operator)) {
-      operator = mrb_fixnum(mrb_operator);
+      operator = mrb_integer(mrb_operator);
     }
   }
 
@@ -349,7 +315,7 @@ mrb_grn_table_delete(mrb_state *mrb, mrb_value self)
 
   mrb_id = grn_mrb_options_get_lit(mrb, mrb_options, "id");
   if (!mrb_nil_p(mrb_id)) {
-    grn_table_delete_by_id(ctx, table, mrb_fixnum(mrb_id));
+    grn_table_delete_by_id(ctx, table, mrb_integer(mrb_id));
     grn_mrb_ctx_check(mrb);
     return mrb_nil_value();
   }
@@ -506,8 +472,6 @@ grn_mrb_table_init(grn_ctx *ctx)
   klass = mrb_define_class_under(mrb, module, "Table", object_class);
   MRB_SET_INSTANCE_TT(klass, MRB_TT_DATA);
 
-  mrb_define_method(mrb, klass, "[]",
-                    mrb_grn_table_array_reference, MRB_ARGS_REQ(1));
   mrb_define_method(mrb, klass, "id?",
                     mrb_grn_table_is_id, MRB_ARGS_REQ(1));
 
