@@ -50,8 +50,8 @@ writer_write(mrb_state *mrb, mrb_value self)
   case MRB_TT_TRUE :
     GRN_OUTPUT_BOOL(GRN_TRUE);
     break;
-  case MRB_TT_FIXNUM :
-    GRN_OUTPUT_INT64(mrb_fixnum(target));
+  case MRB_TT_INTEGER :
+    GRN_OUTPUT_INT64(mrb_integer(target));
     break;
   case MRB_TT_FLOAT :
     GRN_OUTPUT_FLOAT(mrb_float(target));
@@ -135,7 +135,7 @@ writer_close_map(mrb_state *mrb, mrb_value self)
 }
 
 static mrb_value
-writer_write_table_columns(mrb_state *mrb, mrb_value self)
+writer_open_result_set(mrb_state *mrb, mrb_value self)
 {
   grn_ctx *ctx = (grn_ctx *)mrb->ud;
   mrb_value mrb_table;
@@ -143,12 +143,15 @@ writer_write_table_columns(mrb_state *mrb, mrb_value self)
   mrb_int columns_size;
   grn_obj *table;
   grn_obj_format format;
-  int n_hits = 0;
+  mrb_int n_hits = 0;
   int offset = 0;
   int limit = 0;
   int hits_offset = 0;
+  mrb_int n_additional_elements = 0;
 
-  mrb_get_args(mrb, "os", &mrb_table, &columns, &columns_size);
+  mrb_get_args(mrb, "osi|i",
+               &mrb_table, &columns, &columns_size, &n_hits,
+               &n_additional_elements);
 
   table = DATA_PTR(mrb_table);
   GRN_OBJ_FORMAT_INIT(&format, n_hits, offset, limit, hits_offset);
@@ -162,12 +165,22 @@ writer_write_table_columns(mrb_state *mrb, mrb_value self)
       grn_mrb_ctx_check(mrb);
     }
   }
-  GRN_OUTPUT_TABLE_COLUMNS(table, &format);
+  GRN_OUTPUT_RESULT_SET_OPEN(table, &format, n_additional_elements);
   grn_obj_format_fin(ctx, &format);
 
   return mrb_nil_value();
 }
 
+static mrb_value
+writer_close_result_set(mrb_state *mrb, mrb_value self)
+{
+  grn_ctx *ctx = (grn_ctx *)mrb->ud;
+
+  GRN_OUTPUT_RESULT_SET_CLOSE(NULL, NULL);
+
+  return mrb_nil_value();
+}
+
 static mrb_value
 writer_open_table_records(mrb_state *mrb, mrb_value self)
 {
@@ -191,6 +204,7 @@ writer_write_table_records_content_internal(mrb_state *mrb,
   mrb_int columns_size;
   grn_obj *table;
   grn_obj_format format;
+  grn_obj *condition = NULL;
   int n_hits = 0;
   int offset = 0;
   int limit = -1;
@@ -203,12 +217,12 @@ writer_write_table_records_content_internal(mrb_state *mrb,
   if (!mrb_nil_p(mrb_options)) {
     mrb_value mrb_offset = grn_mrb_options_get_lit(mrb, mrb_options, "offset");
     if (!mrb_nil_p(mrb_offset)) {
-      offset = mrb_fixnum(mrb_offset);
+      offset = mrb_integer(mrb_offset);
     }
 
     mrb_value mrb_limit = grn_mrb_options_get_lit(mrb, mrb_options, "limit");
     if (!mrb_nil_p(mrb_limit)) {
-      limit = mrb_fixnum(mrb_limit);
+      limit = mrb_integer(mrb_limit);
     }
 
     mrb_value mrb_auto_flush =
@@ -216,6 +230,12 @@ writer_write_table_records_content_internal(mrb_state *mrb,
     if (!mrb_nil_p(mrb_auto_flush)) {
       auto_flush = mrb_bool(mrb_auto_flush);
     }
+
+    mrb_value mrb_condition =
+      grn_mrb_options_get_lit(mrb, mrb_options, "condition");
+    if (!mrb_nil_p(mrb_condition)) {
+      condition = DATA_PTR(mrb_condition);
+    }
   }
   if (limit < 0) {
     limit = grn_table_size(ctx, table) + limit + 1;
@@ -232,6 +252,9 @@ writer_write_table_records_content_internal(mrb_state *mrb,
       grn_obj_format_fin(ctx, &format);
       grn_mrb_ctx_check(mrb);
     }
+    if (format.expression && condition) {
+      grn_expr_set_condition(ctx, format.expression, condition);
+    }
   }
   if (only_content) {
     grn_ctx_output_table_records_content(ctx, table, &format);
@@ -297,8 +320,10 @@ grn_mrb_writer_init(grn_ctx *ctx)
   mrb_define_method(mrb, klass, "close_map",
                     writer_close_map, MRB_ARGS_NONE());
 
-  mrb_define_method(mrb, klass, "write_table_columns",
-                    writer_write_table_columns, MRB_ARGS_REQ(2));
+  mrb_define_method(mrb, klass, "open_result_set",
+                    writer_open_result_set, MRB_ARGS_ARG(3, 1));
+  mrb_define_method(mrb, klass, "close_result_set",
+                    writer_close_result_set, MRB_ARGS_NONE());
   mrb_define_method(mrb, klass, "open_table_records",
                     writer_open_table_records, MRB_ARGS_REQ(1));
   mrb_define_method(mrb, klass, "write_table_records_content",
