@@ -6,6 +6,8 @@ module Groonga
       Operator::MATCH,
       Operator::NEAR,
       Operator::NEAR2,
+      Operator::NEAR_PHRASE,
+      Operator::ORDERED_NEAR_PHRASE,
       Operator::SIMILAR,
       Operator::PREFIX,
       Operator::SUFFIX,
