@@ -1,7 +1,7 @@
 /* -*- c-basic-offset: 2 -*- */
 /*
-  Copyright(C) 2014-2017 Brazil
-  Copyright(C) 2018-2019 Sutou Kouhei <kou@clear-code.com>
+  Copyright(C) 2014-2017  Brazil
+  Copyright(C) 2018-2020  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -19,13 +19,12 @@
 
 #include "grn.h"
 #include "grn_db.h"
+#include "grn_float.h"
 #include "grn_str.h"
 #include "grn_normalizer.h"
 #include "grn_onigmo.h"
 
 #include <string.h>
-#include <math.h>
-#include <float.h>
 
 static const char *operator_names[] = {
   "push",
@@ -107,10 +106,97 @@ static const char *operator_names[] = {
   "get_member",
   "regexp",
   "fuzzy",
-  "quorum"
+  "quorum",
+  "near_phrase",
+  "ordered_near_phrase",
 };
 
-#define GRN_OP_LAST GRN_OP_QUORUM
+static const char *operator_script_syntaxes[] = {
+  "(nonexistent:push)",
+  "(nonexistent:pop)",
+  "(nonexistent:nop)",
+  "(nonexistent:call)",
+  "(nonexistent:intern)",
+  "(nonexistent:get_ref)",
+  "(nonexistent:get_value)",
+  "&&",
+  "&!",
+  "||",
+  "=",
+  "*=",
+  "/=",
+  "%=",
+  "+=",
+  "-=",
+  "<<=",
+  ">>=",
+  ">>>=",
+  "&=",
+  "^=",
+  "|=",
+  "(nonexistent:jump)",
+  "(nonexistent:cjump)",
+  "(nonexistent:comma)",
+  "|",
+  "^",
+  "&",
+  "~",
+  "==",
+  "!=",
+  "<",
+  ">",
+  "<=",
+  ">=",
+  "(nonexistent:in)",
+  "@",
+  "*N",
+  "(nonexistent:near2)",
+  "*S",
+  "*T",
+  "<<",
+  ">>",
+  ">>>",
+  "+",
+  "-",
+  "*",
+  "/",
+  "%",
+  "(nonexistent:delete)",
+  "++",
+  "--",
+  "++",
+  "--",
+  "!",
+  "(nonexistent:adjust)", /* *>, *< or *~ */
+  "(nonexistent:exact)",
+  "(nonexistent:lcp)",
+  "(nonexistent:partial)",
+  "(nonexistent:unsplit)",
+  "@^",
+  "@$",
+  "(nonexistent:geo_distance1)",
+  "(nonexistent:geo_distance2)",
+  "(nonexistent:geo_distance3)",
+  "(nonexistent:geo_distance4)",
+  "(nonexistent:geo_withinp5)",
+  "(nonexistent:geo_withinp6)",
+  "(nonexistent:geo_withinp8)",
+  "(nonexistent:obj_search)",
+  "(nonexistent:expr_get_var)",
+  "(nonexistent:table_create)",
+  "(nonexistent:table_select)",
+  "(nonexistent:table_sort)",
+  "(nonexistent:table_group)",
+  "(nonexistent:json_put)",
+  "(nonexistent:get_member)",
+  "@~",
+  "(nonexistent:fuzzy)",
+  "*Q",
+  "*NP",
+  "*ONP",
+};
+
+#define GRN_OP_LAST GRN_OP_ORDERED_NEAR_PHRASE
 
 const char *
 grn_operator_to_string(grn_operator op)
@@ -122,6 +208,16 @@ grn_operator_to_string(grn_operator op)
   }
 }
 
+const char *
+grn_operator_to_script_syntax(grn_operator op)
+{
+  if (op >= 0 && op <= GRN_OP_LAST) {
+    return operator_script_syntaxes[op];
+  } else {
+    return "unknown";
+  }
+}
+
 grn_operator_exec_func *
 grn_operator_to_exec_func(grn_operator op)
 {
@@ -191,8 +287,11 @@ grn_operator_to_exec_func(grn_operator op)
   case GRN_DB_UINT64 :\
     r = (x_ == GRN_UINT64_VALUE(y));\
     break;\
+  case GRN_DB_FLOAT32 :\
+    r = grn_float32_is_zero(x_ - GRN_FLOAT32_VALUE(y));\
+    break;\
   case GRN_DB_FLOAT :\
-    r = (fabs(x_ - GRN_FLOAT_VALUE(y)) < DBL_EPSILON);\
+    r = grn_float_is_zero(x_ - GRN_FLOAT_VALUE(y));\
     break;\
   case GRN_DB_SHORT_TEXT :\
   case GRN_DB_TEXT :\
@@ -273,6 +372,9 @@ grn_operator_to_exec_func(grn_operator op)
       case GRN_DB_UINT64 :\
         r = (x_ == GRN_UINT64_VALUE(y));\
         break;\
+      case GRN_DB_FLOAT32 :\
+        r = (x_ == GRN_TIME_PACK(GRN_FLOAT32_VALUE(y), 0));\
+        break;\
       case GRN_DB_FLOAT :\
         r = (x_ == GRN_TIME_PACK(GRN_FLOAT_VALUE(y), 0));\
         break;\
@@ -302,6 +404,44 @@ grn_operator_to_exec_func(grn_operator op)
       DO_EQ_SUB(y, r);\
     }\
     break;\
+  case GRN_DB_FLOAT32 :\
+    {\
+      float x_ = GRN_FLOAT32_VALUE(x);\
+      switch (y->header.domain) {\
+      case GRN_DB_INT32 :\
+        r = ((x_ <= GRN_INT32_VALUE(y)) && (x_ >= GRN_INT32_VALUE(y)));\
+        break;\
+      case GRN_DB_UINT32 :\
+        r = ((x_ <= GRN_UINT32_VALUE(y)) && (x_ >= GRN_UINT32_VALUE(y)));\
+        break;\
+      case GRN_DB_INT64 :\
+      case GRN_DB_TIME :\
+        r = ((x_ <= GRN_INT64_VALUE(y)) && (x_ >= GRN_INT64_VALUE(y)));\
+        break;\
+      case GRN_DB_UINT64 :\
+        r = ((x_ <= GRN_UINT64_VALUE(y)) && (x_ >= GRN_UINT64_VALUE(y)));\
+        break;\
+      case GRN_DB_FLOAT32 :\
+        r = ((x_ <= GRN_FLOAT32_VALUE(y)) && (x_ >= GRN_FLOAT32_VALUE(y)));\
+        break;\
+      case GRN_DB_FLOAT :\
+        r = ((x_ <= GRN_FLOAT_VALUE(y)) && (x_ >= GRN_FLOAT_VALUE(y)));\
+        break;\
+      case GRN_DB_SHORT_TEXT :\
+      case GRN_DB_TEXT :\
+      case GRN_DB_LONG_TEXT :\
+        {\
+          const char *p_ = GRN_TEXT_VALUE(y);\
+          int i_ = grn_atoi(p_, p_ + GRN_TEXT_LEN(y), NULL);\
+          r = (x_ <= i_ && x_ >= i_);\
+        }\
+        break;\
+      default :\
+        r = GRN_FALSE;\
+        break;\
+      }\
+    }\
+    break;\
   case GRN_DB_FLOAT :\
     {\
       double x_ = GRN_FLOAT_VALUE(x);\
@@ -319,6 +459,9 @@ grn_operator_to_exec_func(grn_operator op)
       case GRN_DB_UINT64 :\
         r = ((x_ <= GRN_UINT64_VALUE(y)) && (x_ >= GRN_UINT64_VALUE(y)));\
         break;\
+      case GRN_DB_FLOAT32 :\
+        r = ((x_ <= GRN_FLOAT32_VALUE(y)) && (x_ >= GRN_FLOAT32_VALUE(y)));\
+        break;\
       case GRN_DB_FLOAT :\
         r = ((x_ <= GRN_FLOAT_VALUE(y)) && (x_ >= GRN_FLOAT_VALUE(y)));\
         break;\
@@ -407,11 +550,11 @@ exec_equal(grn_ctx *ctx, grn_obj *x, grn_obj *y)
       for (i = 0; i < x_size; i++) {
         const char *x_value;
         unsigned int x_size;
-        unsigned int x_weight;
+        uint32_t x_weight;
         grn_id x_domain;
         const char *y_value;
         unsigned int y_size;
-        unsigned int y_weight;
+        uint32_t y_weight;
         grn_id y_domain;
 
         x_size = grn_vector_get_element(ctx, x, i, &x_value, &x_weight, &x_domain);
@@ -573,6 +716,9 @@ grn_operator_exec_not_equal(grn_ctx *ctx, grn_obj *x, grn_obj *y)
   case GRN_DB_UINT64 :\
     r = (x_ op GRN_UINT64_VALUE(y));\
     break;\
+  case GRN_DB_FLOAT32 :\
+    r = (x_ op GRN_FLOAT32_VALUE(y));\
+    break;\
   case GRN_DB_FLOAT :\
     r = (x_ op GRN_FLOAT_VALUE(y));\
     break;\
@@ -699,6 +845,9 @@ grn_operator_exec_not_equal(grn_ctx *ctx, grn_obj *x, grn_obj *y)
       case GRN_DB_UINT64 :\
         r = (x_ op GRN_UINT64_VALUE(y));\
         break;\
+      case GRN_DB_FLOAT32 :\
+        r = (x_ op GRN_TIME_PACK(GRN_FLOAT32_VALUE(y), 0));\
+        break;\
       case GRN_DB_FLOAT :\
         r = (x_ op GRN_TIME_PACK(GRN_FLOAT_VALUE(y), 0));\
         break;\
@@ -734,6 +883,12 @@ grn_operator_exec_not_equal(grn_ctx *ctx, grn_obj *x, grn_obj *y)
       DO_COMPARE_SCALAR_SUB(op);\
     }\
     break;\
+  case GRN_DB_FLOAT32 :\
+    {\
+      float x_ = GRN_FLOAT32_VALUE(x);\
+      DO_COMPARE_SCALAR_SUB(op);\
+    }\
+    break;\
   case GRN_DB_FLOAT :\
     {\
       double x_ = GRN_FLOAT_VALUE(x);\
