@@ -1284,16 +1284,12 @@ grn_plugin_clear_error(grn_ctx *ctx)
 void
 grn_plugin_backtrace(grn_ctx *ctx)
 {
-  BACKTRACE(ctx);
 }
 
 void
 grn_plugin_logtrace(grn_ctx *ctx, grn_log_level level)
 {
-  if (level <= GRN_LOG_ERROR) {
-    grn_plugin_backtrace(ctx);
-    LOGTRACE(ctx, level);
-  }
+  grn_ctx_log_back_trace(ctx, level);
 }
 
 struct _grn_plugin_mutex {
@@ -1534,20 +1530,38 @@ grn_plugin_command_create(grn_ctx *ctx,
 }
 
 
+bool
+grn_plugin_proc_get_value_bool(grn_ctx *ctx,
+                               grn_obj *value,
+                               bool default_value,
+                               const char *tag)
+{
+  return grn_proc_get_value_bool(ctx, value, default_value, tag);
+}
+
 int64_t
 grn_plugin_proc_get_value_int64(grn_ctx *ctx,
                                 grn_obj *value,
                                 int64_t default_value_raw,
-                                const char *context)
+                                const char *tag)
 {
-  return grn_proc_get_value_int64(ctx, value, default_value_raw, context);
+  return grn_proc_get_value_int64(ctx, value, default_value_raw, tag);
 }
 
 grn_operator
 grn_plugin_proc_get_value_mode(grn_ctx *ctx,
                                grn_obj *value,
                                grn_operator default_mode,
-                               const char *context)
+                               const char *tag)
 {
-  return grn_proc_get_value_mode(ctx, value, default_mode, context);
+  return grn_proc_get_value_mode(ctx, value, default_mode, tag);
+}
+
+grn_operator
+grn_plugin_proc_get_value_operator(grn_ctx *ctx,
+                                   grn_obj *value,
+                                   grn_operator default_operator,
+                                   const char *tag)
+{
+  return grn_proc_get_value_operator(ctx, value, default_operator, tag);
 }
