@@ -1,7 +1,7 @@
 /* -*- c-basic-offset: 2 -*- */
 /*
-  Copyright(C) 2009-2016 Brazil
-  Copyright(C) 2019 Sutou Kouhei <kou@clear-code.com>
+  Copyright(C) 2009-2016  Brazil
+  Copyright(C) 2019-2020  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -21,6 +21,7 @@
 #include "../grn_ii.h"
 #include "../grn_proc.h"
 #include "../grn_rset.h"
+#include "../grn_posting.h"
 
 #include <groonga/plugin.h>
 
@@ -283,12 +284,15 @@ sequential_fuzzy_search(grn_ctx *ctx, grn_obj *table, grn_obj *column, grn_obj *
         break;
       }
       {
-        grn_posting posting;
+        grn_posting_internal posting = {0};
         posting.rid = heap->nodes[i].id;
         posting.sid = 1;
         posting.pos = 0;
-        posting.weight = max_distance - heap->nodes[i].score + 1;
-        grn_ii_posting_add(ctx, &posting, (grn_hash *)res, op);
+        posting.weight_float = max_distance - heap->nodes[i].score + 1;
+        grn_ii_posting_add_float(ctx,
+                                 (grn_posting *)(&posting),
+                                 (grn_hash *)res,
+                                 op);
       }
     }
     grn_ii_resolve_sel_and(ctx, (grn_hash *)res, op);
@@ -405,11 +409,13 @@ selector_fuzzy_search(grn_ctx *ctx, grn_obj *table, grn_obj *index,
                       int nargs, grn_obj **args,
                       grn_obj *res, grn_operator op)
 {
+  const char *tag = "[fuzzy-search]";
   grn_rc rc = GRN_SUCCESS;
 
   if ((nargs - 1) < 2) {
     GRN_PLUGIN_ERROR(ctx, GRN_INVALID_ARGUMENT,
-                     "fuzzy_search(): wrong number of arguments (%d for 2..)",
+                     "%s wrong number of arguments (%d for 2..)",
+                     tag,
                      nargs - 1);
     rc = ctx->rc;
     goto exit;
@@ -433,47 +439,42 @@ selector_fuzzy_search(grn_ctx *ctx, grn_obj *table, grn_obj *index,
       break;
     case GRN_TABLE_HASH_KEY :
       {
-        grn_hash_cursor *cursor;
-        void *key;
-        grn_obj *value;
-        int key_size;
-        cursor = grn_hash_cursor_open(ctx, (grn_hash *)options,
-                                      NULL, 0, NULL, 0,
-                                      0, -1, 0);
-        if (!cursor) {
-          GRN_PLUGIN_ERROR(ctx, GRN_NO_MEMORY_AVAILABLE,
-                           "fuzzy_search(): couldn't open cursor");
+        int64_t max_distance = data.max_distance;
+        int64_t prefix_length = data.prefix_length;
+        int64_t max_expansion = data.max_expansion;
+        bool with_transposition = false;
+        rc = grn_proc_options_parse(ctx,
+                                    options,
+                                    tag,
+                                    "max_distance",
+                                    GRN_PROC_OPTION_VALUE_INT64,
+                                    &max_distance,
+                                    "prefix_length",
+                                    GRN_PROC_OPTION_VALUE_INT64,
+                                    &prefix_length,
+                                    "max_expansion",
+                                    GRN_PROC_OPTION_VALUE_INT64,
+                                    &max_expansion,
+                                    "with_transposition",
+                                    GRN_PROC_OPTION_VALUE_BOOL,
+                                    &with_transposition,
+                                    NULL);
+        if (rc != GRN_SUCCESS) {
           goto exit;
         }
-        while (grn_hash_cursor_next(ctx, cursor) != GRN_ID_NIL) {
-          grn_hash_cursor_get_key_value(ctx, cursor, &key, &key_size,
-                                        (void **)&value);
-
-          if (key_size == 12 && !memcmp(key, "max_distance", 12)) {
-            data.max_distance = GRN_UINT32_VALUE(value);
-          } else if (key_size == 13 && !memcmp(key, "prefix_length", 13)) {
-            data.prefix_length = GRN_UINT32_VALUE(value);
-          } else if (key_size == 13 && !memcmp(key, "max_expansion", 13)) {
-            data.max_expansion = GRN_UINT32_VALUE(value);
-          } else if (key_size == 18 && !memcmp(key, "with_transposition", 18)) {
-            if (GRN_BOOL_VALUE(value)) {
-              data.flags |= GRN_TABLE_FUZZY_SEARCH_WITH_TRANSPOSITION;
-            }
-          } else {
-            GRN_PLUGIN_ERROR(ctx, GRN_INVALID_ARGUMENT,
-                             "invalid option name: <%.*s>",
-                             key_size, (char *)key);
-            grn_hash_cursor_close(ctx, cursor);
-            goto exit;
-          }
+        data.max_distance = max_distance;
+        data.prefix_length = prefix_length;
+        data.max_expansion = max_expansion;
+        if (with_transposition) {
+          data.flags |= GRN_TABLE_FUZZY_SEARCH_WITH_TRANSPOSITION;
         }
-        grn_hash_cursor_close(ctx, cursor);
       }
       break;
     default :
       GRN_PLUGIN_ERROR(ctx, GRN_INVALID_ARGUMENT,
-                       "fuzzy_search(): "
+                       "%s "
                        "3rd argument must be integer or object literal: <%.*s>",
+                       tag,
                        (int)GRN_TEXT_LEN(options),
                        GRN_TEXT_VALUE(options));
       goto exit;
