@@ -33,12 +33,91 @@ grn_type_id_is_number_family(grn_ctx *ctx, grn_id id)
   return (GRN_DB_INT8 <= id && id <= GRN_DB_FLOAT) || (id == GRN_DB_FLOAT32);
 }
 
+bool
+grn_type_id_is_float_family(grn_ctx *ctx, grn_id id)
+{
+  switch (id) {
+  case GRN_DB_FLOAT32 :
+  case GRN_DB_FLOAT :
+    return true;
+  default :
+    return false;
+  }
+}
+
 grn_bool
 grn_type_id_is_text_family(grn_ctx *ctx, grn_id id)
 {
   return GRN_DB_SHORT_TEXT <= id && id <= GRN_DB_LONG_TEXT;
 }
 
+size_t
+grn_type_id_size(grn_ctx *ctx, grn_id id)
+{
+  switch (id) {
+  case GRN_DB_BOOL :
+    return sizeof(bool);
+  case GRN_DB_INT8 :
+    return sizeof(int8_t);
+  case GRN_DB_UINT8 :
+    return sizeof(uint8_t);
+  case GRN_DB_INT16 :
+    return sizeof(int16_t);
+  case GRN_DB_UINT16 :
+    return sizeof(uint16_t);
+  case GRN_DB_INT32 :
+    return sizeof(int32_t);
+  case GRN_DB_UINT32 :
+    return sizeof(uint32_t);
+  case GRN_DB_INT64 :
+    return sizeof(int64_t);
+  case GRN_DB_UINT64 :
+    return sizeof(uint64_t);
+  case GRN_DB_FLOAT32 :
+    return sizeof(float);
+  case GRN_DB_FLOAT :
+    return sizeof(double);
+  case GRN_DB_TIME :
+    return sizeof(int64_t);
+  case GRN_DB_SHORT_TEXT :
+    return GRN_TYPE_SHORT_TEXT_SIZE;
+  case GRN_DB_TEXT :
+    return GRN_TYPE_TEXT_SIZE;
+  case GRN_DB_LONG_TEXT :
+    return GRN_TYPE_LONG_TEXT_SIZE;
+  case GRN_DB_TOKYO_GEO_POINT :
+  case GRN_DB_WGS84_GEO_POINT :
+    return sizeof(grn_geo_point);
+  default :
+    {
+      GRN_API_ENTER;
+      size_t size = 0;
+      if (grn_type_id_is_builtin(ctx, id)) {
+        grn_obj *obj = grn_ctx_at(ctx, id);
+        if (grn_obj_is_type(ctx, obj)) {
+          size = grn_type_size(ctx, obj);
+        }
+        grn_obj_unref(ctx, obj);
+      } else {
+        /*
+          We need to check the ID is a ID for table:
+
+            grn_obj *obj = grn_ctx_at(ctx, id);
+            if (grn_obj_is_table(ctx, obj)) {
+              size = sizeof(grn_id);
+            }
+            grn_obj_unref(ctx, obj);
+
+          But we want to reduce grn_ctx_at() call for performance.
+          So we assume the ID is a ID for table here.
+        */
+        size = sizeof(grn_id);
+      }
+      GRN_API_RETURN(size);
+    }
+  }
+}
+
 grn_obj *
 grn_type_create(grn_ctx *ctx, const char *name, unsigned int name_size,
                 grn_obj_flags flags, unsigned int size)
