@@ -1,5 +1,5 @@
 # Copyright(C) 2014-2020  Sutou Kouhei <kou@clear-code.com>
-# Copyright(C) 2020  Horimoto Yasuhiro <horimoto@clear-code.com>
+# Copyright(C) 2020-2021  Horimoto Yasuhiro <horimoto@clear-code.com>
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -25,6 +25,7 @@ class PackagesGroongaOrgPackageTask < PackageTask
 
   def define
     super
+    define_windows_task
     define_release_tasks
     define_ubuntu_tasks
   end
@@ -61,6 +62,34 @@ class PackagesGroongaOrgPackageTask < PackageTask
     super
   end
 
+  def enable_windows?
+    true
+  end
+
+  def windows_dir
+    "windows"
+  end
+
+  def windows_targets
+    return [] unless enable_windows?
+
+    targets = (ENV["WINDOWS_TARGETS"] || "").split(",")
+    return targets unless targets.empty?
+
+    windows_targets_default
+  end
+
+  def define_windows_task
+    namespace :windows do
+      desc "Clean files for Windows"
+      task :clean do
+        rm_rf("#{windows_dir}/repositories")
+      end
+    end
+    desc "Release files for Windows"
+    task windows: ["windows:clean"]
+  end
+
   def use_built_package?
     false
   end
@@ -75,19 +104,33 @@ class PackagesGroongaOrgPackageTask < PackageTask
 
   def download_packages(target_namespace)
     base_dir = __send__("#{target_namespace}_dir")
-    repositories_dir = "#{base_dir}/repositories"
+    case target_namespace
+    when :windows
+      repositories_dir = "#{base_dir}/repositories/windows/#{@package}"
+    else
+      repositories_dir = "#{base_dir}/repositories"
+    end
     mkdir_p(repositories_dir)
+    download_dir = "#{base_dir}/tmp/downloads/#{@version}"
+    mkdir_p(download_dir)
     __send__("#{target_namespace}_targets").each do |target|
       url = built_package_url(target_namespace, target)
-      archive = File.expand_path(url.split("/").last)
-      rm_f(archive)
-      sh("wget", url)
-      cd(repositories_dir) do
-        sh("tar",
-           "xf", archive,
-           "--strip-components=#{built_package_n_split_components}")
+      archive = download(url, download_dir)
+      case target_namespace
+      when :apt, :yum
+        cd(repositories_dir) do
+          sh("tar",
+             "xf", archive,
+             "--strip-components=#{built_package_n_split_components}")
+        end
+      when :windows
+        cd(repositories_dir) do
+          cp(archive, ".")
+          archive_base_name = File.basename(archive)
+          latest_link_base_name = archive_base_name.gsub(@version, "latest")
+          ln_sf(archive_base_name, latest_link_base_name)
+        end
       end
-      rm_f(archive)
     end
   end
 
@@ -101,7 +144,7 @@ class PackagesGroongaOrgPackageTask < PackageTask
   end
 
   def define_release_tasks
-    [:apt, :yum].each do |target_namespace|
+    [:apt, :yum, :windows].each do |target_namespace|
       tasks = []
       namespace target_namespace do
         enabled = __send__("enable_#{target_namespace}?")
@@ -112,14 +155,14 @@ class PackagesGroongaOrgPackageTask < PackageTask
           task :download do
             download_packages(target_namespace) if enabled
           end
-          tasks << ["#{target_namespace}:download"]
+          tasks << "#{target_namespace}:download"
         end
 
         desc "Release #{target_namespace} packages"
         task :release do
           release(target_namespace) if enabled
         end
-        tasks << ["#{target_namespace}:release"]
+        tasks << "#{target_namespace}:release"
       end
       task target_namespace => tasks
     end
