@@ -21,46 +21,59 @@ module Groonga
 
       def run_body(input)
         output_columns = input[:output_columns] || "_key, *"
+        is_stream_output = stream_output?(input)
 
         context = ExecuteContext.new(input)
         begin
-          executor = Executor.new(context)
-          executor.execute
-
-          result_sets = context.result_sets
-          n_elements = 1 # for columns
-          result_sets.each do |result_set|
-            n_elements += result_set.size
+          limit = context.limit
+          if limit < 0 and limit != -1
+            message =
+              "[logical_range_filter] negative limit must be -1: #{limit}"
+            raise InvalidArgument, message
           end
 
-          writer.array("RESULTSET", n_elements) do
-            first_result_set = result_sets.first
-            if first_result_set
-              writer.write_table_columns(first_result_set, output_columns)
-            end
-            limit = context.limit
-            if limit < 0
-              n_records = result_sets.inject(0) do |n, result_set|
-                n + result_set.size
-              end
-              limit = n_records + limit + 1
+          n_elements = 0
+          is_first = true
+          executor = Executor.new(context)
+          executor.execute do |result_set|
+            n_elements += result_set.size
+
+            if is_first
+              writer.open_result_set(result_set, output_columns, -1)
+              writer.open_table_records(-1)
+              is_first = false
             end
+
             options = {}
-            result_sets.each do |result_set|
-              options[:limit] = limit
-              writer.write_table_records(result_set, output_columns, options)
+            options[:limit] = limit
+            options[:auto_flush] = true if is_stream_output
+            writer.write_table_records_content(result_set,
+                                               output_columns,
+                                               options)
+            writer.flush if is_stream_output
+            if limit != -1
               limit -= result_set.size
               break if limit <= 0
             end
           end
-          query_logger.log(:size, ":", "output(#{n_elements - 1})")
+          unless is_first
+            writer.close_table_records
+            writer.close_result_set
+          end
+          query_logger.log(:size, ":", "output(#{n_elements})")
         ensure
           context.close
         end
       end
 
       private
+      def stream_output?(input)
+        context.command_version >= 3
+      end
+
       def cache_key(input)
+        return nil if stream_output?(input)
+
         key = "logical_range_filter\0"
         key << "#{input[:logical_table]}\0"
         key << "#{input[:shard_key]}\0"
@@ -76,6 +89,8 @@ module Groonga
         key << "#{input[:use_range_index]}\0"
         key << "#{input[:post_filter]}\0"
         key << "#{input[:sort_keys]}\0"
+        key << "#{context.command_version}\0"
+        key << "#{context.output_type}\0"
         dynamic_columns = DynamicColumns.parse("[logical_range_filter]",
                                                input)
         key << dynamic_columns.cache_key
@@ -96,11 +111,11 @@ module Groonga
         attr_reader :dynamic_columns
         attr_accessor :current_offset
         attr_accessor :current_limit
-        attr_reader :result_sets
-        attr_reader :temporary_tables
         attr_reader :threshold
         attr_reader :large_shard_threshold
         attr_reader :time_classify_types
+        attr_reader :result_sets
+        attr_reader :referred_objects
         def initialize(input)
           @input = input
           @use_range_index = parse_use_range_index(@input[:use_range_index])
@@ -119,17 +134,44 @@ module Groonga
           @current_limit = @limit
 
           @result_sets = []
-          @temporary_tables = []
+          @temporary_tables_stack = []
 
           @threshold = compute_threshold
           @large_shard_threshold = compute_large_shard_threshold
 
           @time_classify_types = detect_time_classify_types
+
+          @referred_objects = []
+        end
+
+        def temporary_tables
+          @temporary_tables_stack.last
+        end
+
+        def push
+          @temporary_tables_stack << []
+        end
+
+        def shift
+          temporary_tables = @temporary_tables_stack.shift
+          temporary_tables.each do |table|
+            table.close
+          end
+          temporary_tables.clear
         end
 
         def close
-          @temporary_tables.each do |table|
-            table.close
+          until @temporary_tables_stack.empty?
+            shift
+          end
+          @result_sets.clear
+          @referred_objects.each(&:unref)
+          @enumerator.unref
+        end
+
+        def consume_result_sets
+          while (result_set = @result_sets.shift)
+            yield(result_set)
           end
         end
 
@@ -215,9 +257,14 @@ module Groonga
 
         def execute
           first_shard = nil
+          have_result_set = false
           each_shard_executor do |shard_executor|
             first_shard ||= shard_executor.shard
             shard_executor.execute
+            @context.consume_result_sets do |result_set|
+              have_result_set = true
+              yield(result_set)
+            end
             break if @context.current_limit.zero?
           end
           if first_shard.nil?
@@ -228,14 +275,16 @@ module Groonga
               "shard_key: <#{enumerator.shard_key_name}>"
             raise InvalidArgument, message
           end
-          if @context.result_sets.empty?
+          unless have_result_set
             result_set = HashTable.create(:flags => ObjectFlags::WITH_SUBREC,
                                           :key_type => first_shard.table)
+            @context.push
             @context.temporary_tables << result_set
             targets = [[result_set]]
             @context.dynamic_columns.apply_initial(targets)
             @context.dynamic_columns.apply_filtered(targets)
-            @context.result_sets << result_set
+            yield(result_set)
+            @context.shift
           end
         end
 
@@ -252,6 +301,7 @@ module Groonga
             executors = []
             previous_executor = nil
             enumerator.send(each_method) do |shard, shard_range|
+              @context.push
               current_executor = ShardExecutor.new(@context, shard, shard_range)
               if previous_executor
                 previous_executor.next_executor = current_executor
@@ -261,10 +311,18 @@ module Groonga
               previous_executor = current_executor
             end
             executors << previous_executor if previous_executor
-            executors.each(&block)
+            return if executors.empty?
+            executors.each_with_index do |executor, i|
+              yield(executor)
+              # Keep the previous data for window function
+              @context.shift if i > 0
+            end
+            @context.shift
           else
             enumerator.send(each_method) do |shard, shard_range|
+              @context.push
               yield(ShardExecutor.new(@context, shard, shard_range))
+              @context.shift
             end
           end
         end
@@ -560,21 +618,7 @@ module Groonga
               end
             end
           else
-            apply_targets = []
-            if @previous_executor
-              @previous_executor.add_filtered_stage_context(apply_targets)
-            end
-            @filtered_result_sets.each do |result_set|
-              apply_targets << [result_set]
-            end
-            if @next_executor
-              @next_executor.add_filtered_stage_context(apply_targets)
-            end
-            options = {}
-            if @context.need_look_ahead?
-              options[:normal] = false
-            end
-            @context.dynamic_columns.apply_filtered(apply_targets, options)
+            apply_filtered_dynamic_columns
             @filtered_result_sets.each do |result_set|
               sort_result_set(result_set)
             end
@@ -617,8 +661,10 @@ module Groonga
                                                            @target_range)
           @expression_builder.filter = @filter
           index_info = @shard_key.find_index(Operator::LESS)
-          if index_info and use_range_index?
-            @range_index = index_info.index
+          if index_info
+            index = index_info.index
+            @context.referred_objects << index
+            @range_index = index if use_range_index?
           end
 
           if @context.dynamic_columns.have_initial?
@@ -640,6 +686,15 @@ module Groonga
           @window = detect_window
         end
 
+        def compute_limit(n_source_records)
+          current_limit = @context.current_limit
+          if current_limit < 0
+            n_source_records
+          else
+            current_limit
+          end
+        end
+
         def decide_use_range_index(use, reason, line, method)
           message = "[logical_range_filter]"
           if use
@@ -781,6 +836,17 @@ module Groonga
                                           __LINE__, __method__)
           end
 
+          max_n_unmatched_records =
+              compute_max_n_unmatched_records(max_n_records,
+                                              required_n_records)
+          if estimated_n_records >= max_n_unmatched_records
+            reason = "the max number of unmatched records (#{max_n_unmatched_records}) "
+            reason << "<= "
+            reason << "the number of estimated records (#{estimated_n_records})"
+            return decide_use_range_index(true, reason,
+                                          __LINE__, __method__)
+          end
+
           hit_ratio = estimated_n_records / max_n_records.to_f
           use_range_index_by_hit_ratio = (hit_ratio >= threshold)
           if use_range_index_by_hit_ratio
@@ -949,7 +1015,9 @@ module Groonga
 
         def filter_by_range(range_index, min, min_border, max, max_border)
           lexicon = range_index.domain
+          @context.referred_objects << lexicon
           data_table = range_index.range
+          @context.referred_objects << data_table
           flags = build_range_search_flags(min_border, max_border)
 
           result_set = HashTable.create(:flags => ObjectFlags::WITH_SUBREC,
@@ -962,13 +1030,8 @@ module Groonga
                            :flags => flags) do |table_cursor|
             options = {
               :offset => @context.current_offset,
+              :limit => compute_limit(data_table.size),
             }
-            current_limit = @context.current_limit
-            if current_limit < 0
-              options[:limit] = data_table.size
-            else
-              options[:limit] = current_limit
-            end
             max_n_unmatched_records =
               compute_max_n_unmatched_records(data_table.size,
                                               options[:limit])
@@ -1043,17 +1106,19 @@ module Groonga
           flags
         end
 
-        def compute_max_n_unmatched_records(data_table_size, limit)
-          max_n_unmatched_records = limit * 100
-          max_n_sample_records = data_table_size
+        def compute_max_n_unmatched_records(max_n_records, limit)
+          return limit if max_n_records.zero?
+
+          scale = Math.log(max_n_records) ** 2
+          max_n_unmatched_records = limit * 2 * scale
+          max_n_sample_records = max_n_records
           if max_n_sample_records > 10000
-            sample_ratio = 1 / (Math.log(data_table_size) ** 2)
-            max_n_sample_records = (max_n_sample_records * sample_ratio).ceil
+            max_n_sample_records = max_n_sample_records / scale
           end
           if max_n_unmatched_records > max_n_sample_records
             max_n_unmatched_records = max_n_sample_records
           end
-          max_n_unmatched_records
+          max_n_unmatched_records.ceil
         end
 
         def filter_table
@@ -1078,6 +1143,31 @@ module Groonga
           @filtered_result_sets << result_set
         end
 
+        def apply_filtered_dynamic_columns
+          return unless @context.dynamic_columns.have_filtered?
+
+          apply_targets = []
+          if @previous_executor
+            @previous_executor.add_filtered_stage_context(apply_targets)
+          end
+          @filtered_result_sets = @filtered_result_sets.collect do |result_set|
+            if result_set == @shard.table
+              result_set = result_set.select_all
+              @temporary_tables << result_set
+            end
+            apply_targets << [result_set]
+            result_set
+          end
+          if @next_executor
+            @next_executor.add_filtered_stage_context(apply_targets)
+          end
+          options = {}
+          if @context.need_look_ahead?
+            options[:normal] = false
+          end
+          @context.dynamic_columns.apply_filtered(apply_targets, options)
+        end
+
         def sort_result_set(result_set)
           unless @post_filter.nil?
             result_set = apply_post_filter(result_set)
