@@ -1,4 +1,5 @@
 #!/usr/bin/env ruby
+# -*- coding: utf-8 -*-
 
 if ARGV.size != 2
   puts "Usage: #{$0} SOURCE_DIR DEST_DIR"
@@ -46,6 +47,14 @@ def fix_html_link(html, language)
   end
 end
 
+def add_language_annotation_to_source_label(html, language)
+  return html unless language == "ja"
+  html.gsub(/>(ソースコードを表示)</) do
+    label = $1
+    ">#{label}（英語）<"
+  end
+end
+
 def fix_js_link(js, language)
   fix_link_path(js)
 end
@@ -120,6 +129,24 @@ def insert_facebook_html(html, language)
   html
 end
 
+def insert_google_analytics_html(html)
+  html.gsub(/<head>/) do
+    <<-HTML
+<head>
+
+    <!-- Global site tag (gtag.js) - Google Analytics -->
+    <script async src="https://www.googletagmanager.com/gtag/js?id=UA-7532323-1"></script>
+    <script>
+      window.dataLayer = window.dataLayer || [];
+      function gtag(){dataLayer.push(arguments);}
+      gtag('js', new Date());
+
+      gtag('config', 'UA-7532323-1');
+    </script>
+    HTML
+  end
+end
+
 source_dir, dest_dir = ARGV
 
 source_dir = Pathname.new(source_dir)
@@ -147,6 +174,8 @@ language_dirs.each do |language_dir|
         content = fix_link(content, extension, language)
         if extension == "html"
           content = insert_facebook_html(content, language)
+          content = insert_google_analytics_html(content)
+          content = add_language_annotation_to_source_label(content, language)
         end
         dest_path.open("wb") do |dest|
           dest.print(content.strip)
